package edu.uoc.pfc.lcamps.persistenciaFW.admin.connection;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.connection.impl.ConnectionFWImpl;
import edu.uoc.pfc.lcamps.persistenciaFW.config.PropertiesConfigFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;

/**
 * Classe que ens servirà per obtenir la connexió a la BBDD
 * @author Llorenç Camps
 * 12/12/2013
 */
public class ConnectionFWFactory {

	private static final Logger log = Logger.getLogger( ConnectionFWFactory.class );

	private static ConnectionFWFactory instance;
	private ConnectionFW conFW;
	
	// Constructors
	private ConnectionFWFactory(){super();}
	
	public static synchronized ConnectionFWFactory getInstance() throws DBException{
		if( instance == null ){
			instance = new ConnectionFWFactory();
			instance.config();
		}
		return instance;
	}
	
	/**
	 * Obtenim la connexió a la BBDD
	 * @return
	 * @throws DBException
	 */
	public Connection getConnection() throws DBException{
		try {
			return DriverManager.getConnection( conFW.getUrl(), conFW.getUsuari(), conFW.getPassword() );
		} catch (SQLException e) {
			log.error( "Error obtenint la connexió: " + e.getMessage() );
			throw new DBException( ErrorConstants.ERROR_GET_CONNECTION_CODI, ErrorConstants.ERROR_GET_CONNECTION_MSG, e );
		}
	}
	
	// Getters && Setters
	
	public ConnectionFW getConFW() {
		return conFW;
	}

	public void setConFW(ConnectionFW conFW) {
		this.conFW = conFW;
	}
	
	// Mètodes privats
	
	/**
	 * Configuració de la connexió
	 */
	private void config() throws DBException{
		// Obtenim els les dades de configuració del fitxer
		PropertiesConfigFactory propConfig = PropertiesConfigFactory.getInstance();
		String nomDriver = propConfig.getDriver();
		String tipusDriver = propConfig.getDriverType();
		String url = propConfig.getURLDB();
		String usuari = propConfig.getUser();
		String pass = propConfig.getPassword();
		instance.setConFW( new ConnectionFWImpl( nomDriver, tipusDriver, url, usuari, pass ) );
	}
}
