package edu.uoc.pfc.lcamps.persistenciaFW.admin.connection.impl;

import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.connection.ConnectionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;

/**
 * Classe que ens servirà per obtenir el driver per tal de connectar-nos a la BBDD
 * @author Llorenç Camps
 * 12/12/2013
 */
public class ConnectionFWImpl implements ConnectionFW{

	private static final Logger log = Logger.getLogger( ConnectionFWImpl.class );

	// atributs
	private String nomDriver;
	private String tipusDriver;
	private String url;
	private String usuari;
	private String password;
	private Driver driver;
	
	// Constructor
	
	public ConnectionFWImpl( String nomDriver, String tipusDriver, String url, String usuari, String password ) throws DBException{
		this.nomDriver = nomDriver;
		this.tipusDriver = tipusDriver;
		this.url = url;
		this.usuari = usuari;
		this.password = password;
		// inicialitzem la connexio de la BD
		this.inicialitzacio();
	}
		
	// Getters && Setters
	public String getNomDriver() {
		return nomDriver;
	}

	public void setNomDriver(String nomDriver) {
		this.nomDriver = nomDriver;
	}

	public String getTipusDriver() {
		return tipusDriver;
	}

	public void setTipusDriver(String tipusDriver) {
		this.tipusDriver = tipusDriver;
	}

	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getUsuari() {
		return usuari;
	}

	public void setUsuari(String usuari) {
		this.usuari = usuari;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}
	
	public Driver getDriver() {
		return driver;
	}

	public void setDriver(Driver driver) {
		this.driver = driver;
	}
	
	// Mètodes privats
	
	/**
	 * Mètode que inicialitzar la connexió de base de dades i dependrà del motor de la base de dades
	 * 
	 */
	private void inicialitzacio() throws DBException{
		// Obtenim el driver		
		try {
			driver = (Driver) Class.forName( nomDriver ).newInstance();
		} catch (InstantiationException e) {
			log.error( "Error inicialitzant el driver: " + e.getMessage() );
			throw new DBException( ErrorConstants.ERROR_INIT_DRIVER_CONNECTION_CODI, ErrorConstants.ERROR_INIT_DRIVER_CONNECTION_MSG, e );
		} catch (IllegalAccessException e) {
			log.error( "Error inicialitzant el driver: " + e.getMessage() );			
			throw new DBException( ErrorConstants.ERROR_INIT_DRIVER_CONNECTION_CODI, ErrorConstants.ERROR_INIT_DRIVER_CONNECTION_MSG, e );
		} catch (ClassNotFoundException e) {
			log.error( "Error inicialitzant el driver: " + e.getMessage() );
			throw new DBException( ErrorConstants.ERROR_INIT_DRIVER_CONNECTION_CODI, ErrorConstants.ERROR_INIT_DRIVER_CONNECTION_MSG, e );
		}
		
		// Registrem el driver
		try {
			DriverManager.registerDriver( driver );
		} catch (SQLException e) {
			log.error( "Error registrant el driver: " + e.getMessage() );
			throw new DBException( ErrorConstants.ERROR_REGISTER_DRIVER_CONNECTION_CODI, ErrorConstants.ERROR_REGISTER_DRIVER_CONNECTION_MSG, e );
		}
	}
}
