package edu.uoc.pfc.lcamps.persistenciaFW.admin.pojo;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Column;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Entity;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Id;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Sequence;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Table;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;

/**
 * Classe POJO que equival a una taula de la base de dades amb tota la informació
 * @author Llorenç Camps
 * 12/12/2013
 */
public class RegisterDBVO {
	
	// Atributs
	private String nameTable;
	private Object object;
	private List<ColumnDBVO> columns;
	private String primaryKey;
	
	// Constructors
	
	public RegisterDBVO(){
		this.columns = new ArrayList<ColumnDBVO>();
	}
		
	public RegisterDBVO( Object object ) throws DBException{
		this();
		this.object = object;
		if( this.object == null ) throw new DBException( ErrorConstants.ERROR_OBJECT_NULL_CODI, ErrorConstants.ERROR_OBJECT_NULL_MSG );
		// Hem de parsejar l'objecte que ens passen a un objecte registre amb totes les metadades
		this.parserObjectToRegisterDBVO( this.object );
	}
	
	/**
	 * Obtenim la columna de la clau primària
	 * TODO al igual que el mètode del parser s'ha de revisar el cas de les claus forànees
	 * @return
	 */
	public ColumnDBVO getColumnPrimaryKey(){		
		ColumnDBVO out = new ColumnDBVO();
		for( ColumnDBVO aux : this.columns ){
			if ( aux.isPrimaryKey() ) {
				out.setPrimaryKey( true );
				out.setMethod( aux.getMethod() );
				out.setName( aux.getName() );
				out.setSequence( aux.getSequence() );
				out.setType( aux.getType() );
				return out; 
			}
		}
		return out;
	}
	
	// Getters && Setters
	
	public String getNameTable() {
		return nameTable;
	}

	public void setNameTable(String nameTable) {
		this.nameTable = nameTable;
	}

	public Object getRegister() {
		return object;
	}

	public void setRegister(Object register) {
		this.object = register;
	}
	
	public List<ColumnDBVO> getColumns() {
		return columns;
	}

	public void setColumns(List<ColumnDBVO> columns) {
		this.columns = columns;
	}

	public String getPrimaryKey() {
		return primaryKey;
	}

	public void setPrimaryKey(String primaryKey) {
		this.primaryKey = primaryKey;
	}
	
	// Mètodes privats
	
	/**
	 * Obtindrem l'objecte per reflection
	 */
	private void parserObjectToRegisterDBVO( Object obj ) throws DBException{
		Class<?> classe = obj.getClass();
		
		if( !classe.isAnnotationPresent( Entity.class ) ){
			throw new DBException( ErrorConstants.ERROR_REGISTRE_NO_ENTITAT_ANNOTATION_CODI, ErrorConstants.ERROR_REGISTRE_NO_ENTITAT_ANNOTATION_MSG );
		}
		
		if ( classe.isAnnotationPresent( Table.class ) ){
			Table taula = classe.getAnnotation( Table.class );
			// Obtenim el nom de la taula 
	        this.nameTable = taula.name();
		} else {
			// Si no es troba com a nom per defecte es posa el nom de la classe		
			this.nameTable = classe.toString();
		}
		
		// Obtenim els mètodes
        Method[] methods = classe.getMethods();

	    for( Method method : methods ) {
	    	
	    	ColumnDBVO columnVO = null;
	        
	    	// Mirem si és de tipus columna, i en cas afirmatiu fem la transformació
	    	if( method.isAnnotationPresent( Column.class ) ){
	    		columnVO = this.columnAnnotationToColumnDBVO( method );
	        }
	    	
	    	// Mirem si és de tipus Identificador, i en cas afirmatiu fem la transformació
	        if( method.isAnnotationPresent( Id.class ) ){
	        	columnVO = this.idAnnotationToColumnDBVO( method );
	        }
	        
	        // Mirem si és de tipus sequencia, i en cas afirmatiu afegim el nom de la sequencia
	        if( method.isAnnotationPresent( Sequence.class ) ) {
	        	Sequence sq = method.getAnnotation( Sequence.class);
	        	columnVO.setSequence( sq.key() );
	        }
	        
	        // Per acabar afegim la columna al llistat 
	        if( columnVO != null ) this.columns.add( columnVO );
	    }
	}
	
	/**
	 * 
	 * @param method
	 * @return
	 */
	private ColumnDBVO columnAnnotationToColumnDBVO( Method method ){
		Column column = method.getAnnotation( Column.class );
		ColumnDBVO out = new ColumnDBVO();
		out.setName( column.name() );
		out.setType( column.type() );
		out.setMethod( method.getName() );
		out.setPrimaryKey( false );
		return out;
	}
	
	/**
	 * 
	 * @param method
	 * @return
	 */
	private ColumnDBVO idAnnotationToColumnDBVO( Method method ){
		// TODO Revisar si hi ha Foreign keys
		Id id = method.getAnnotation( Id.class );
    	this.primaryKey = id.name();
    	ColumnDBVO out = new ColumnDBVO();
    	out.setName(id.name());
    	out.setType(id.type());
    	out.setMethod(method.getName());
    	out.setPrimaryKey( true );
    	return out;
	}
}