package edu.uoc.pfc.lcamps.persistenciaFW.admin.session.impl;

import java.sql.ResultSet;
import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.connection.ConnectionFWFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.pojo.RegisterDBVO;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.TransactionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.impl.TransactionFWImpl;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.SQLConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.persistenciaFW.utils.SQLUtils;

/**
 * Classe que ens servirà per la gestió de les sessions
 * @author Llorenç Camps
 * 12/12/2013
 */
public class SessionFWImpl implements SessionFW {
	
	TransactionFW transactionFW = null;

	// Constructors
	
	public SessionFWImpl(){}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW#newTransaction()
	 */
	public TransactionFW newTransaction() throws DBException{
		return newTransaction( true );
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW#newTransaction(boolean)
	 */
	public TransactionFW newTransaction( boolean autoCommit ) throws DBException{
		this.setTransactionFW( new TransactionFWImpl( autoCommit ) );
		return this.getTransactionFW();
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW#closeTransaction()
	 */
	public void closeTransaction() throws DBException{
		this.getTransactionFW().close();
		this.setTransactionFW( null );
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW#insert(java.lang.Object)
	 */
	@Override
	public int insert(Object obj) throws DBException {
		int id = -1;
		// Obtenim l'objecte
		RegisterDBVO registerVO = new RegisterDBVO( obj );
		
		// Generem la Query SQL d'insert
		String query = SQLUtils.getQueryInsert( registerVO );
		
		// Revisem en quin tipus de BBDD estem utilitzant
		if( ConnectionFWFactory.getInstance().getConFW().getUrl().indexOf( SQLConstants.ORACLE ) != -1 ){
			// Creem la query per obtenir el nou id per l'insert
			String queryNum = SQLConstants.QRY_SELECT_COUNT.replaceFirst( "%s", registerVO.getPrimaryKey() );
			id = this.getTransactionFW().executeCount( queryNum );
			// Fem el replace per l'id obtingut
			query = query.replaceFirst( SQLConstants.ID_AUTO_INCREMENTAL_ORACLE, String.valueOf( id ) );
			// Executem la query
			this.getTransactionFW().executeQuery( query );
		}else if( ConnectionFWFactory.getInstance().getConFW().getUrl().indexOf( SQLConstants.MYSQL ) != -1 ){
			// En MYSQL genera automaticament l'id i dspres es pot consultar
			// Executem la query
			this.getTransactionFW().executeQuery( query );
			// Obtenim l'id generat
			id = this.getTransactionFW().executeCount( SQLConstants.QRY_ID_MYSQL );
		}else{
			// Executem la query
			this.getTransactionFW().executeQuery( query );
		}
		// Retornem l'id
		return id;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW#update(java.lang.Object)
	 */
	@Override
	public void update(Object obj) throws DBException {
		// Obtenim l'objecte
		RegisterDBVO registerVO = new RegisterDBVO( obj );
		// Generem la Query SQL d'update
		String query = SQLUtils.getQueryUpdate( registerVO );
		// Executem la query
		this.getTransactionFW().executeQuery( query );
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW#delete(java.lang.Object)
	 */
	@Override
	public void delete(Object obj) throws DBException {
		// Obtenim l'objecte
		RegisterDBVO registerVO = new RegisterDBVO( obj );
		// Generem la Query SQL de delete
		String query = SQLUtils.getQueryDelete( registerVO );
		// Executem la query
		this.getTransactionFW().executeQuery( query );
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW#findListFromObj(java.lang.Object)
	 */
	@Override
	public List<Object> findListFromObj(Object obj) throws DBException {
		// Obtenim l'objecte
		RegisterDBVO registerVO = new RegisterDBVO( obj );
		// Generem la query per cercar per id
		String query = SQLUtils.getQueryFindById( registerVO );
		// Executem la query
		ResultSet rs = this.getTransactionFW().executeQueryData( query );
		// Transformem el ResultSet a un llistat d'objectes
		return SQLUtils.transformResultSetToObjectList( rs, obj );
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW#findListFromQuery(java.lang.String, java.lang.Object)
	 */
	@Override
	public List<Object> findListFromQuery(String query, Object objReturn) throws DBException {
		// Executem la query
		ResultSet rs = this.getTransactionFW().executeQueryData( query );
		// Transformem el ResultSet a un llistat d'objectes
		return SQLUtils.transformResultSetToObjectList( rs, objReturn );
	}
	
	// Getters && Setters
	
	public void setTransactionFW(TransactionFW transactionFW) {
		this.transactionFW = transactionFW;
	}

	public TransactionFW getTransactionFW() {
		return transactionFW;
	}	
}