package edu.uoc.pfc.lcamps.persistenciaFW.config;

import java.io.InputStream;
import java.util.Properties;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.constants.ConfigConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.persistenciaFW.utils.FileUtils;

/**
 * Classe que gestiona les properties del fitxer de configuració
 * @author Llorenç Camps
 * 12/12/2013
 */
public class PropertiesConfigFactory{
	
	private static final Logger log = Logger.getLogger( PropertiesConfigFactory.class );

	private static PropertiesConfigFactory instance = null;
	
	private Properties cfg = null;
	
	// Constructors
	private PropertiesConfigFactory(){ super(); }
	
	public static synchronized PropertiesConfigFactory getInstance() throws DBException {		  
		if( instance == null ){
	    	instance = new PropertiesConfigFactory();
	    	instance.config();
		}
	    return instance;
	}
	
	/** 
	 * Retorna el contingut del driver de la BD
	 * @return
	 * @throws DBException
	 */
	public String getDriver() throws DBException{
		return this.getProperty( ConfigConstants.FW_DB_DRIVER );
	}
	
	/** 
	 * Retorna el contingut del tipus del driver de la BD
	 * @return
	 * @throws DBException
	 */
	public String getDriverType() throws DBException{
		return this.getProperty( ConfigConstants.FW_DB_TIPUS_DRIVER );
	}
	
	/** 
	 * Retorna el contingut de la url de connexió a la BD
	 * @return
	 * @throws DBException
	 */
	public String getURLDB() throws DBException{
		return this.getProperty( ConfigConstants.FW_DB_URL );
	}
	
	/** 
	 * Retorna el contingut del usuari de la BD
	 * @return
	 * @throws DBException
	 */
	public String getUser() throws DBException{
		return this.getProperty( ConfigConstants.FW_DB_USER );
	}
	
	/** 
	 * Retorna el contingut del password de la BD
	 * @return
	 * @throws DBException
	 */
	public String getPassword() throws DBException{
		return this.getProperty( ConfigConstants.FW_DB_PASS );
	}
	
	/** 
	 * Retorna el contingut del package del provider
	 * @return
	 * @throws DBException
	 */
	public String getBeansPackage() throws DBException{
		return this.getProperty( ConfigConstants.FW_DB_BEANS_PACKAGE );
	}
	
	/**
	 * Retorna el path base a on generara els beans
	 * @return
	 * @throws DBException
	 */
	public String getBeansBaseDir() throws DBException{
		return this.getProperty( ConfigConstants.FW_DB_BEANS_BASEDIR );
	}
	
	// Mètodes privats

	/**
	 * Carrega l'arxiu de properties
	 */
	private void config() throws DBException {
		try{
			InputStream is = this.getPropertiesInputStream();
			//InputStream is = FileUtils.getFileAsStream( ConfigConstants.FW_CONFIG_FILE );
		    cfg = new Properties();
		    cfg.load(is);
		}catch (Exception e){
			throw new DBException( ErrorConstants.ERROR_CONFIG_FILE_CODI, ErrorConstants.ERROR_CONFIG_FILE_MSG, e );
		}
	}
	
	/**
	 * Obté un arxiu com Stream al que es té accés des del classloader
	 * @return Stream de l'arxiu o null si no ho troba
	 */
	private InputStream getPropertiesInputStream() {
		InputStream is = null;
	    String filename = ConfigConstants.FW_CONFIG_FILE;
	    if( filename!=null && filename.trim().length() >  0){
	    	is = getClass().getClassLoader().getResourceAsStream( filename );
	    	if (is==null) {
	    		log.warn( "L'arxiu [" + filename + "] no existeix" );
	    		is = FileUtils.getFileAsStreamFromClassLoader( filename );
	    	}
	    }
	    return is;
	}
	
	/**
	 * Donada una property aquest retorna el seu contingut
	 * @param propName
	 * @return el contingut de la property
	 * @throws DBException
	 */
	private String getProperty( String propName ) throws DBException {
	    if( this.cfg == null ){
	      throw new DBException( ErrorConstants.ERROR_CONFIG_FILE_CONFIGURAT_CODI, ErrorConstants.ERROR_CONFIG_FILE_CONFIGURAT_MSG );
	    }
	    String out = this.cfg.getProperty( propName );    
	    return out!=null ? out.trim() : out;
	}
}
