package edu.uoc.pfc.lcamps.persistenciaFW.generator;

import java.io.BufferedWriter;
import java.io.File;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.connection.ConnectionFWFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.config.PropertiesConfigFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.GeneratorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.persistenciaFW.utils.FileUtils;
import edu.uoc.pfc.lcamps.persistenciaFW.utils.GeneratorUtils;
import edu.uoc.pfc.lcamps.persistenciaFW.utils.StringUtilsFW;

/**
 * Classe que ens servirà per generar els beans de la base de dades
 * @author Llorenç Camps
 * 13/12/2013
 */
public class GenerateBeansFW {
	
	private static GenerateBeansFW instance = null;
	
	Connection conection = null;
	
	// Constructors
	private GenerateBeansFW(){}
	
	public static synchronized GenerateBeansFW getInstance() throws DBException {
		if( instance == null ){
			instance = new GenerateBeansFW();
			instance.conection = ConnectionFWFactory.getInstance().getConnection();
		}
		return instance;
	}
	
	/**
	 * Genera els beans de base de dades
	 * @throws DBException
	 */
	public void generateDBBeans() throws DBException{
		BufferedWriter out = null;
		try{
			String baseDir = PropertiesConfigFactory.getInstance().getBeansBaseDir();
			String packageName = PropertiesConfigFactory.getInstance().getBeansPackage();
			
			// Obtenim el llistat de taules 
			for( String tableAux: this.getTableList( conection ) ){
				// Obtenim tota la info de la taula segons el tipus connexió
				GeneratorTableDBFW genTable = (GeneratorTableDBFW) getDataFromSchema( ConnectionFWFactory.getInstance().getConFW().getTipusDriver(), tableAux );	
				// Esborrem el director per tornar a generar les classes
				File folderBase = new File( FileUtils.getFolderBase( baseDir, packageName ) );
				FileUtils.deleteFolder( folderBase );
				
				// Generem els POJOs per cadascuna de les taules
				String nomFitxer = genTable.getFileName();
				
				// Afegim la Primary Key
				genTable.addPKey();
				
				// Afegim la informació de les columnes
				genTable.addColumnData();
				
				// Creem el fitxer
				out = FileUtils.createFile( folderBase, nomFitxer );
				
				// Escriu la classe java en funció de la BBDD
				GeneratorUtils.writeHeader( out, genTable );
				GeneratorUtils.writeConstructor( out, genTable);
				GeneratorUtils.writeSetGet( out, genTable);
				GeneratorUtils.writeToString( out, genTable );
				GeneratorUtils.writeEnd(out);				
			}
		}catch(Exception e){
			throw new DBException( ErrorConstants.ERROR_GENERATE_BEANS_CODI, ErrorConstants.ERROR_GENERATE_BEANS_MSG + e.getMessage(), e );
		}finally{
			try{out.close();}catch(Exception ignored){out = null;}
		}
	}
		
	// Mètodes privats
	
	/**
	 * Obtenim el llistat de taules de l'esquema
	 */
	private List<String> getTableList( Connection con ) throws Exception{
		List<String> table = new ArrayList<String>();
		DatabaseMetaData meta = con.getMetaData();
	    ResultSet res = meta.getTables( null, null, null, new String[]{ "TABLE" } );
	    while (res.next()) {
	    	// Obtenim el nom de la taula
	    	table.add( res.getString( "TABLE_NAME" ) );
	    }
	    return table;
	}
	
	/**
	 * Generem la info de taula segons el tipus de connexió
	 * @param driverType
	 * @param tableName
	 * @return
	 * @throws Exception
	 */	
	private GeneratorTableDBFW getDataFromSchema(String driverType, String tableName) throws Exception {
		// Fer  la creacio de la taula per refection en funcio de la configuració
		GeneratorTableDBFW genTable = ( GeneratorTableDBFW ) Class.forName( GeneratorUtils.getClassNameFromDriver( driverType ) ).newInstance();
		genTable.setTableName( tableName );
		genTable.setFileName( getFileName( tableName ) );
		genTable.setClassName( getClassName( tableName ) );
		return genTable;
	}
	
	/**
	 * Obtenim el nom del fitxer a partir del nom de la taula
	 * @param tableName
	 * @return
	 */
	private String getFileName( String tableName ) {
		return StringUtilsFW.upperCamelCase( tableName ).concat( GeneratorConstants.SUFIX_GENERATE_CLASS ).concat( GeneratorConstants.JAVA_EXTENSION );
	}
	
	/**
	 * Obtenim el nom del fitxer a partir del nom de la taula
	 * @param tableName
	 * @return
	 */
	private String getClassName( String tableName ) {
		return StringUtilsFW.upperCamelCase( tableName ).concat( GeneratorConstants.SUFIX_GENERATE_CLASS );
	}
	
	// Main per fer proves
	
	public static void main (String args[]) throws Exception{
		System.out.println( " Creant els beans d'acces a la BBDD..." );
		GenerateBeansFW.getInstance().generateDBBeans();
		System.out.println( " Beans d'acces a la BBDD creats" );
	}
}