package edu.uoc.pfc.lcamps.persistenciaFW.generator.impl;

import java.sql.ResultSet;

import edu.uoc.pfc.lcamps.persistenciaFW.constants.GeneratorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.generator.GeneratorColumnDBFW;

/**
 * Classe que ens servirà per la gestió de la generació de la columna
 * @author Llorenç Camps
 * 13/12/2013
 */
public class GeneratorColumnDBFWImpl implements GeneratorColumnDBFW{

	// Atributs
	private String name;
	private String nameType;
	private int type;
	private int precision;
	private int size;	  
	private boolean nullable;
	private boolean autoincrement;
	
	// Constructors
	public GeneratorColumnDBFWImpl(){}
	
	public GeneratorColumnDBFWImpl( String name, int type, int precision ) {
		this.name = name;
		this.type = type;
		this.precision = precision;

		switch (this.type){
			case -1:
				this.type = 22;
				break;
		    case -2:
		    	this.type = 18;
		    	break;
		    case -3:
		    	this.type = 19;
		    	break;
		    case -4:
		    	this.type = 19;
		    	break;
		    case -5:
		    	this.type = 3;
		    	break;
		    case -6:
		    	this.type = 2;
		    	break;
		    case -7:
		    	this.type = 16;
		    	break;
		    case 1111:
		    	this.type = 13;
		    	break;
		    case 91:
		    	this.type = 11;
		    	break;
		    case 92:
		    	this.type = 14;
		    	break;
		    case 93:
		    	this.type = 14;
		    	break;
		}

		if( ( this.type < 1 ) || ( this.type > 24 ) ){
			// Tipus per defecte STRING
		  	this.type = 12;
		}
	}
	
	/**
	 * Fem parse del resultSet
	 * @param localResultSet
	 * @return
	 * @throws Exception
	 */
	public Object parseResult( ResultSet rs ) throws Exception {
		GeneratorColumnDBFWImpl columnData = ( GeneratorColumnDBFWImpl ) this.getClass().newInstance();
		columnData.name = rs.getString( GeneratorConstants.METADADA_COLUMN_NAME_INDEX );
		columnData.type = rs.getInt( GeneratorConstants.METADADA_COLUMN_DATATYPE_INDEX );
		columnData.nameType = rs.getString( GeneratorConstants.METADADA_COLUMN_TYPENAME_INDEX );
		columnData.size = rs.getInt( GeneratorConstants.METADADA_COLUMN_SIZE_INDEX );
		columnData.precision = rs.getInt( GeneratorConstants.METADADA_COLUMN_DECIMALDIGITS_INDEX ); 
		columnData.nullable = rs.getBoolean( GeneratorConstants.METADADA_COLUMN_ISNULLABLE_INDEX );
		columnData.autoincrement = rs.getBoolean( GeneratorConstants.METADADA_COLUMN_ISAUTOINCREMENT_INDEX );
	       
	    return columnData;
	}
	
	/**
	 * Obtenim el tipus
	 * @return
	 */
	public String getJavaType(){
	    String javaType = null;

	    switch( this.type ){
	    	case 1:		    
		    case 2:
		    	javaType = "byte";
		    	break;
		    case 3:
		    	javaType = "long";
		    	break;
		    case 4:
		    	javaType = "int";
		    	break;
		    case 5:
		    	javaType = "short";
		    	break;
		    case 6:
		    case 7:
		    	javaType = "float";
		    	break;
		    case 8:
		    	javaType = "double";
		    	break;
		    case 9:
		    case 10:
		    	javaType = "java.math.BigDecimal";
		    	break;
		    case 11:
		    	javaType = "java.util.Date";
		    	break;
		    case 12:
		    case 13:		    
		    case 14:
		    	javaType = "Timestamp";
		    	break;
		    case 15:
		    	javaType = "Time";
		    	break;
		    case 16:
		    	javaType = "boolean";
		    	break;
		    case 17:
		    case 18:
		    case 19:
		    	javaType = "byte[]";
		    	break;
		    case 20:
		    	javaType = "null";
		    	break;
		    case 21:
		    case 22:
		    	javaType = "String";
		    	break;
		    default:
		    	javaType = "String";
		}										

	    return javaType;
	}
	
	// Getters && Setters
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public String getNameType() {
		return nameType;
	}
	
	public void setNameType(String nameType) {
		this.nameType = nameType;
	}
	
	public int getType() {
		return type;
	}
	
	public void setType(int type) {
		this.type = type;
	}
	
	public int getPrecision() {
		return precision;
	}
	
	public void setPrecision(int precision) {
		this.precision = precision;
	}
	
	public int getSize() {
		return size;
	}
	
	public void setSize(int size) {
		this.size = size;
	}
	
	public boolean isNullable() {
		return nullable;
	}
	
	public void setNullable(boolean nullable) {
		this.nullable = nullable;
	}
	
	public boolean isAutoincrement() {
		return autoincrement;
	}
	
	public void setAutoincrement(boolean autoincrement) {
		this.autoincrement = autoincrement;
	}	
}