package edu.uoc.pfc.lcamps.persistenciaFW.generator.impl;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.connection.ConnectionFWFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.config.PropertiesConfigFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.persistenciaFW.generator.GeneratorColumnDBFW;
import edu.uoc.pfc.lcamps.persistenciaFW.generator.GeneratorTableDBFW;
import edu.uoc.pfc.lcamps.persistenciaFW.generator.provider.mysql.TableDBMysqlImpl;
import edu.uoc.pfc.lcamps.persistenciaFW.utils.StringUtilsFW;

/**
 * Classe que ens servirà per la gestió de la generació de la taula
 * @author Llorenç Camps
 * 13/12/2013
 */
public abstract class GeneratorTableDBFWImpl implements GeneratorTableDBFW{
	
	private static final Logger log = Logger.getLogger( GeneratorTableDBFWImpl.class );
	
	// Metadades de la BBDD
	private DatabaseMetaData metadataDB;
	private String metadataVersion;
	private String metadataDriverName;

	// Mapping Java - BBDD
	private String packageName;
	private String baseDirectory;
	private String className;
	private String fileName;
	private List<GeneratorColumnDBFW> columns;
	private List<String> keys;
	  
	// Taula
	private String tableName;
	private String tableAlias;
	private int numFields;
	
	public Connection connection;
	
	// Constructors
	
	public GeneratorTableDBFWImpl() throws Exception{
		connection = ConnectionFWFactory.getInstance().getConnection();
		this.init( connection );
	}
	
	public GeneratorTableDBFWImpl( Connection con ) throws Exception{
		this.init( con );
	}
	
	// Mètodes abstractes que implementaran cada classe segons el provider (MySQL, Oracle)
	
	public abstract String getPK();
	
	public abstract void addPKey() throws DBException;
	
	public abstract void addColumnData() throws DBException;
	
	// Altres mètodes
	
	/**
	 * Mirem si té PK
	 * @return
	 */
	public boolean hasPK(){
		boolean hasPK = false;
	    this.setKeys( new Vector<String>() );
	    String schema = getPK();
	    try {
	    	String clau = null;
	    	ResultSet rs = this.getMetadataDB().getPrimaryKeys( null, schema, this.getTableName() );
	    	while( rs.next() ){
	    		clau = rs.getString(4);
	    		this.getKeys().add( clau );
	    		hasPK = true;
	    	}
	      	rs.close();
	    }catch (SQLException e){
	    	log.error( "Error obtenint les claus de la taula " + this.getTableName() + " : " + e.getMessage() );
	    }
	    return hasPK;
	}

	// Getters && Setters
	  
	public DatabaseMetaData getMetadataDB() {
		return metadataDB;
	}

	public void setMetadataDB(DatabaseMetaData metadataDB) {
		this.metadataDB = metadataDB;
	}

	public String getMetadataVersion() {
		return metadataVersion;
	}

	public void setMetadataVersion(String metadataVersion) {
		this.metadataVersion = metadataVersion;
	}

	public String getMetadataDriverName() {
		return metadataDriverName;
	}

	public void setMetadataDriverName(String metadataDriverName) {
		this.metadataDriverName = metadataDriverName;
	}

	public String getPackageName() {
		return packageName;
	}

	public void setPackageName(String packageName) {
		this.packageName = packageName;
	}

	public String getBaseDirectory() {
		return baseDirectory;
	}

	public void setBaseDirectory(String baseDirectory) {
		this.baseDirectory = baseDirectory;
	}

	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

	public String getFileName() {
		return fileName;
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	public List<GeneratorColumnDBFW> getColumns() {
		return columns;
	}

	public void setColumns(List<GeneratorColumnDBFW> columns) {
		this.columns = columns;
	}

	public List<String> getKeys() {
		return keys;
	}

	public void setKeys(List<String> keys) {
		this.keys = keys;
	}

	public String getTableName() {
		return tableName;
	}

	public void setTableName(String tableName) {
		this.tableName = tableName;
	}

	public String getTableAlias() {
		if( !StringUtilsFW.isNullOrEmpty( tableAlias ) ){
			return tableAlias;
		}
		return tableName;
	}

	public void setTableAlias(String tableAlias) {
		this.tableAlias = tableAlias;
	}

	public int getNumFields() {
		return numFields;
	}

	public void setNumFields(int numFields) {
		this.numFields = numFields;
	}
	
	public Connection getConnection() {
		return connection;
	}

	public void setConnection(Connection connection) {
		this.connection = connection;
	}
	
	// Mètodes privats
	
	private void init(Connection con) throws Exception {
		this.packageName = PropertiesConfigFactory.getInstance().getBeansPackage();
		this.baseDirectory = PropertiesConfigFactory.getInstance().getBeansBaseDir();
		  		  
		// Omplim les metadades
		this.metadataDB = con.getMetaData();
		this.metadataVersion = con.getMetaData().getDatabaseProductVersion();
		this.metadataDriverName = con.getMetaData().getDriverName();
	}
	
	// Main per fer proves
	
	public static void main(String args[]) throws Exception {
		GeneratorTableDBFWImpl gt = new TableDBMysqlImpl();		
		gt.connection.close();
		System.out.println( "Fi" );
	}
}