package edu.uoc.pfc.lcamps.persistenciaFW.generator.provider.oracle;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.GeneratorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.persistenciaFW.generator.GeneratorColumnDBFW;
import edu.uoc.pfc.lcamps.persistenciaFW.generator.impl.GeneratorTableDBFWImpl;

/**
 * Classe que fa els mappings entre la taula de BBDD i la classe java
 * @author Llorenç Camps
 * 14/12/2013
 */
public class TableDBOraclelImpl extends GeneratorTableDBFWImpl{

	private static final Logger log = Logger.getLogger( TableDBOraclelImpl.class );
	
	// Constructor
	public TableDBOraclelImpl() throws Exception{ super();}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.generator.impl.GeneratorTableDBFWImpl#getPK()
	 */
	@Override
	public String getPK() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.generator.impl.GeneratorTableDBFWImpl#addPKey()
	 */
	@Override
	public void addPKey() throws DBException {
		ResultSet rs = null;
	    try{
	    	this.setKeys( new ArrayList<String>() );
	    	rs = this.getMetadataDB().getPrimaryKeys( null, GeneratorConstants.SYMBOL_LIKE, this.getTableName() );
	    	int numColumnes = 0;
	    	while( rs.next() ){
	    		String columna = rs.getString( GeneratorConstants.METADADA_COLUMN_NAME_INDEX );
	    		this.getKeys().add(columna);
	    		numColumnes++;
	    	}
	      
	    	if (numColumnes==0) throw new DBException( ErrorConstants.ERROR_PK_CODI, ErrorConstants.ERROR_UNDEFINED_PK_MSG );
	    	
	    }catch (SQLException e){
	    	log.error( "Error obtenint les claus de la taula " + this.getTableName() , e );
	    	throw new DBException( ErrorConstants.ERROR_PK_CODI, ErrorConstants.ERROR_PK_MSG + e.getMessage(), e );
	    }finally{
	    	try{ rs.close();}catch(Exception ignored){rs = null;}
	    }
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.generator.impl.GeneratorTableDBFWImpl#addColumnData()
	 */
	@Override
	public void addColumnData() throws DBException {
	    ResultSet rs = null;
	    try{
	    	this.setColumns( new ArrayList<GeneratorColumnDBFW>() );
	    	String user = null;
	    	rs = this.getMetadataDB().getColumns( null, user, this.getTableName().toUpperCase(), GeneratorConstants.SYMBOL_LIKE );
	 	    while( rs.next() ){    	
	    		String nomColumna = rs.getString( GeneratorConstants.METADADA_COLUMN_NAME_INDEX );
	    		ColumnDBOracleImpl columnData = null;
	    		int i, j;
	    		
	    		i = rs.getInt( GeneratorConstants.METADADA_COLUMN_DATATYPE_INDEX );
	    		j = rs.getInt( GeneratorConstants.METADADA_COLUMN_SIZE_INDEX );
	    		
	    		if( rs.getInt( GeneratorConstants.METADADA_COLUMN_DECIMALDIGITS_INDEX ) > 0 ) i = 8;
	    		
	    		columnData = ( ColumnDBOracleImpl) new ColumnDBOracleImpl( nomColumna, i, j );
	    		
	    		this.getColumns().add( columnData ); 
	    	}
	    	this.setNumFields( this.getColumns().size() );
	    } catch (Exception e) {
	    	log.error( "Error obtenint les columnes: " + e.getMessage() ); 
			throw new DBException( ErrorConstants.ERROR_GET_METADATA_CODI, ErrorConstants.ERROR_GET_METADATA_MSG, e );
		}finally{ 
	    	try{rs.close();}catch(Exception ignored){rs = null;}
	    }	    
	}
}