package edu.uoc.pfc.lcamps.persistenciaFW.utils;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;

import org.apache.log4j.Logger;

/**
 * Classe que conté les funcions d'utilitats per generar fitxers
 * @author Llorenç Camps
 * 14/12/2013
 */
public class FileUtils {
	
	private static final Logger log = Logger.getLogger( FileUtils.class );
	
	/**
	 * Obté un arxiu com Stream al que es té accés des del classloader del current Thread
	 * @param filename Nom  de l'arxiu
	 * @return Stream de l'arxiu o null si no ho troba
	 */
	public static InputStream getFileAsStreamFromClassLoader( String filename ) {
		InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream( filename );
	    if( is == null ){
	      log.warn( "L'arxiu [" + filename + "] no existeix" );
	    }
	    return is;
	}
	
	/**
	 * Obté un arxiu com Stream buscant-ho a través del seu path absolut
	 * @param path Path absolut de l'arxiu
	 * @return Stream de l'arxiu o null si no ho troba
	 */
	public static InputStream getFileAsStream(String path) {
	    FileInputStream fis = null;

	    try{
	      fis = new FileInputStream( path );
	    }catch ( FileNotFoundException e ){
	      log.error( "L'arxiu [" + path + "] no existeix", e );
	    }
	    return fis;
	}

	/**
	 * Esborrem tot el contingut d'una carpeta
	 * @param folder
	 * @return
	 * @throws IOException
	 */
	public static boolean deleteFolder(File folder) throws IOException{
		if( folder.exists() ){
			File[] files = folder.listFiles();
		    if( files != null ){
		    	for( int i=0; i < files.length; i++ ){
		    		if(files[i].isDirectory() ){
		    			deleteFolder( files[i] );
		            }else {
		            	files[i].delete();
		            }
		        }
		    }
		}
		return( folder.delete() );
	}
	
	/**
	 * Obtenim la carpeta base
	 * @param basedir
	 * @param packageName
	 * @return
	 */
	public static String getFolderBase( String basedir, String packageName ){
		return basedir + File.separator + packageToFile( packageName );
	}
	
	/**
	 * 
	 * @param packageName
	 * @return
	 */
	public static String packageToFile( String packageName ){
	    return StringUtilsFW.replaceString( packageName, ".", File.separator );
	}
	
	/**
	 * Creem un fitxer a partir de la carpeta i el seu nom
	 * @param folder
	 * @param fileName
	 * @return
	 * @throws IOException
	 */
	public static BufferedWriter createFile(File folder, String fileName) throws IOException{
		try{
			if( !folder.exists() ) folder.mkdirs();
	    }catch (Exception e){
	    	e.printStackTrace();
	    }
	    String nom = folder.getAbsolutePath() + File.separator + fileName ;
	    return new BufferedWriter( new FileWriter( nom ) );	
	}
  
}