package edu.uoc.pfc.lcamps.persistenciaFW.utils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Column;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Entity;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Id;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.Mapper;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;

/**
 * Classe que conté les funcions d'utilitats sobre els resultSets
 * @author Llorenç Camps
 * 13/12/2013
 */
public class ResultSetUtils<E> {
	
	private static final Logger log = Logger.getLogger( ResultSetUtils.class );
	
	/**
	 * 
	 * @param rs
	 * @param obj
	 * @return
	 * @throws DBException
	 */
	@SuppressWarnings("unchecked")
	public List<E> transformResultSetToObjList ( ResultSet rs, @SuppressWarnings("rawtypes") Class classe ) throws DBException{
		List<E> out = new ArrayList<E>();
		
		try{
			if( rs != null ){
				// Revisem que la classe sigui una Entity o un Mapper
				if( classe.isAnnotationPresent( Entity.class ) || classe.isAnnotationPresent( Mapper.class ) ){
					// Obtenim la metadata del rs
					ResultSetMetaData rsmd = rs.getMetaData();
					// get all the attributes of outputClass
					Method[] methods = classe.getMethods();
					while( rs.next() ) {
						E bean = ( E )classe.newInstance();
						for( int i = 0; i < rsmd.getColumnCount(); i++ ) {
							// Obte el nom de la columna SQL
							String columnName = rsmd.getColumnLabel( i + 1 );
							// Obte el valor de la columna SQL
							Object columnValue = rs.getObject( i + 1 );
							
							for( Method methodAux : methods ){
								if( methodAux.isAnnotationPresent( Column.class ) || methodAux.isAnnotationPresent( Id.class ) ){
									// Obtenim l'atribut associat al getter
									String attribute = methodAux.getName().substring(3,4).toLowerCase().concat( methodAux.getName().substring( 4 ) );
									// Cas d'una columna
									Column columnAux = methodAux.getAnnotation( Column.class);
									if( columnAux != null && columnAux.name().equalsIgnoreCase( columnName ) && columnValue != null ) {
										BeanUtils.setProperty( bean, attribute, columnValue );
										log.debug(" Afegeix el valor " + columnValue +  " a l'atribut: " + attribute );
										break;
									}
									// Cas de la columna que es PK
									Id id = methodAux.getAnnotation(Id.class);
									if( id != null && id.name().equalsIgnoreCase( columnName ) && columnValue != null ){
										BeanUtils.setProperty( bean, attribute, columnValue );
										log.debug(" Afegeix el valor " + columnValue +  " a l'atribut: " + attribute );
										break;
									}
								}
							}
						}
						// Afegim a la llista
						out.add( bean );
					}
				}else {
					// és un altre tipus de classe
					throw new DBException( ErrorConstants.ERROR_ANNOTATION_CLASS_MAPPING_CODI, ErrorConstants.ERROR_ANNOTATION_CLASS_MAPPING_MSG );
				}			
			}else return null;	
		}catch(SQLException e) {
			throw new DBException( ErrorConstants.ERROR_SQL_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		} catch (InstantiationException e) {
			throw new DBException( ErrorConstants.ERROR_INSTANTIATION_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		} catch (IllegalAccessException e) {
			throw new DBException( ErrorConstants.ERROR_ACCES_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		} catch (InvocationTargetException e) {
			throw new DBException( ErrorConstants.ERROR_INVOCATION_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}
		
		return out;
	}

}
