package edu.uoc.pfc.lcamps.persistenciaFW.utils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.connection.ConnectionFWFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.pojo.ColumnDBVO;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.pojo.RegisterDBVO;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.GeneratorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.SQLConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;

/**
 * Classe que conté les funcions d'utilitats per generar les querys a la BBDD
 * @author Llorenç Camps
 * 13/12/2013
 */
public class SQLUtils {
	
	private static Logger log = Logger.getLogger(SQLUtils.class.getName());
	
	private static final HashMap<String,String> sqlTokens;
	private static Pattern sqlTokenPattern;

	static
	{           
	    String[][] search_regex_replacement = new String[][]
	    {
	    		//search string     search regex        sql replacement regex
	            {   "\u0000"    ,       "\\x00"     ,       "\\\\0"     },
	            {   "'"         ,       "'"         ,       "\''"     },
	            {   "\""        ,       "\""        ,       "\\\\\""    },
	            {   "\b"        ,       "\\x08"     ,       "\\\\b"     },
	            {   "\n"        ,       "\\n"       ,       "\\\\n"     },
	            {   "\r"        ,       "\\r"       ,       "\\\\r"     },
	            {   "\t"        ,       "\\t"       ,       "\\\\t"     },
	            {   "\u001A"    ,       "\\x1A"     ,       "\\\\Z"     },
	            {   "\\"        ,       "\\\\"      ,       "\\\\\\\\"  }
	    };

	    sqlTokens = new HashMap<String,String>();
	    String patternStr = "";
	    for (String[] srr : search_regex_replacement)
	    {
	        sqlTokens.put(srr[0], srr[2]);
	        patternStr += (patternStr.isEmpty() ? "" : "|") + srr[1];            
	    }
	    sqlTokenPattern = Pattern.compile('(' + patternStr + ')');
	}
	
	/**
	 * Generem la query d'insert a partir de l'objecte RegisterDBVO
	 * @param registerVO
	 * @return
	 * @throws DBException
	 */
	public static String getQueryInsert( RegisterDBVO registerVO ) throws DBException {
		try{
			String values = "";
			StringBuffer sb = new StringBuffer( SQLConstants.INSERT );
			sb.append( registerVO.getNameTable().toUpperCase() );
			
			int i=0;
			// Anem recorrent les columnes per anar afegint en l'insert
			for( ColumnDBVO aux : registerVO.getColumns() ){
				// Mirem si es PK i s'autogenera a traves de la sequencia
				if( aux.isPrimaryKey() && !"".equals( aux.getSequence() ) ){
					if( ConnectionFWFactory.getInstance().getConFW().getUrl().indexOf( SQLConstants.ORACLE ) != -1 ){
						// Es una clau primaria autogenerada 
						if( i > 0 ){
							sb.append( SQLConstants.COMMA );
							values = values.concat( SQLConstants.COMMA );
						}else sb.append( SQLConstants.OPEN_BRACKET );
							
						sb.append( aux.getName().toUpperCase() );
						i++;
						values = values.concat( SQLConstants.ID_AUTO_INCREMENTAL_ORACLE );
					}
				}else{				
					// Cas que no és PK 
					if( i>0 ){
						sb.append( SQLConstants.COMMA );
						values = values.concat( SQLConstants.COMMA );
					}else sb.append( SQLConstants.OPEN_BRACKET );
					
					sb.append( aux.getName().toUpperCase() );
					i++;
					
					Method method = registerVO.getRegister().getClass().getMethod( aux.getMethod(), null );
					Object valueObj = (Object) method.invoke( registerVO.getRegister(), null );
					String result = "";
					
					if( valueObj != null ) {
						// Obtenim el valor
						result = getResultValue( valueObj, aux );
						// Afegim el resultat
						values = values.concat( result );
					} else {
						values = values + "null";
					}
				}
			}
			sb.append( SQLConstants.VALUES ).append(values).append( SQLConstants.CLOSE_BRACKET );
			log.debug( "Query Insert [" + sb.toString() + "]" );
			return sb.toString();
		
		}catch(IllegalArgumentException e){
			throw new DBException( ErrorConstants.ERROR_ARGUMENT_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(IllegalAccessException e){
			throw new DBException( ErrorConstants.ERROR_ACCES_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(InvocationTargetException e){
			throw new DBException( ErrorConstants.ERROR_INVOCATION_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(SecurityException e){
			throw new DBException( ErrorConstants.ERROR_SECURITY_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(NoSuchMethodException e){
			throw new DBException( ErrorConstants.ERROR_METHOD_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}
	}
	
	/**
	 * Generem la query d'update a partir de l'objecte RegisterDBVO
	 * @param registerVO
	 * @return
	 * @throws DBException
	 */
	public static String getQueryUpdate( RegisterDBVO registerVO ) throws DBException {
		try {
			StringBuffer sb = new StringBuffer( SQLConstants.UPDATE );
			sb.append( registerVO.getNameTable().toUpperCase() ).append( SQLConstants.SET );
			
			int i=0;
			String clauseWhere = SQLConstants.WHERE;
			
			// Anem recorrent les columnes per anar afegint a l'update
			for( ColumnDBVO aux : registerVO.getColumns() ){

				Method method = registerVO.getRegister().getClass().getMethod( aux.getMethod(), null );
				Object valueObj = (Object) method.invoke( registerVO.getRegister(), null );
				String result = "";
								
				if( valueObj != null ){
					 
					if ( i>0 )sb.append( SQLConstants.COMMA );
					
					result = getResultValue( valueObj, aux);
					
					// Si és primary Key sera el where de la query
					if( aux.isPrimaryKey() ){
						clauseWhere += aux.getName() + SQLConstants.EQUALS + result;
					}else{
						sb.append( aux.getName() ).append( SQLConstants.EQUALS ).append( result );
						i++;
					}
				}
			}
			sb.append( clauseWhere );
			log.debug( "Query Update [" + sb.toString() + "]" );
			return sb.toString();
			
		}catch(IllegalArgumentException e){
			throw new DBException( ErrorConstants.ERROR_ARGUMENT_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(IllegalAccessException e){
			throw new DBException( ErrorConstants.ERROR_ACCES_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(InvocationTargetException e){
			throw new DBException( ErrorConstants.ERROR_INVOCATION_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(SecurityException e){
			throw new DBException( ErrorConstants.ERROR_SECURITY_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(NoSuchMethodException e){
			throw new DBException( ErrorConstants.ERROR_METHOD_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}
	}
	
	/**
	 * Generem la query de delete a partir de l'objecte RegisterDBVO
	 * @param registerVO
	 * @return
	 * @throws DBException
	 */
	public static String getQueryDelete( RegisterDBVO registerVO ) throws DBException {
		try {
			StringBuffer sb = new StringBuffer( SQLConstants.DELETE );
			sb.append( registerVO.getNameTable().toUpperCase() );
			
			// Anem recorrent les columnes per anar afegint al delete
			for( ColumnDBVO aux : registerVO.getColumns() ){
				
				// Mirem si es PK
				if( aux.isPrimaryKey() ){
					sb.append( SQLConstants.WHERE ).append( aux.getName().toUpperCase() ).append( SQLConstants.EQUALS );
					
					Method method = registerVO.getRegister().getClass().getMethod( aux.getMethod(), null );
					Object objValue = (Object) method.invoke( registerVO.getRegister(), null );
					
					String result = getResultValue( objValue, aux );
					
					sb.append(result);
				}
			}
			log.debug( "Query Delete [" + sb.toString() + "]" );
			return sb.toString();
		}catch(IllegalArgumentException e){
			throw new DBException( ErrorConstants.ERROR_ARGUMENT_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(IllegalAccessException e){
			throw new DBException( ErrorConstants.ERROR_ACCES_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(InvocationTargetException e){
			throw new DBException( ErrorConstants.ERROR_INVOCATION_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(SecurityException e){
			throw new DBException( ErrorConstants.ERROR_SECURITY_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(NoSuchMethodException e){
			throw new DBException( ErrorConstants.ERROR_METHOD_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}
	}
	
	/**
	 * Generem la query de cerca per id a partir de l'objecte RegisterDBVO
	 * @param registerVO
	 * @return
	 * @throws DBException
	 */
	public static String getQueryFindById( RegisterDBVO registerVO ) throws DBException {	
		StringBuffer sb = new StringBuffer( SQLConstants.SELECT_ALL );
		sb.append( registerVO.getNameTable().toUpperCase() );
		// Afegim la primary key en la query
		sb.append( addClauseWherePK( registerVO ) );	
			
		return sb.toString();
	}
	
	/**
	 * Obtenim el tipus de la columna
	 * @param nomTipus
	 * @return
	 */
	public static String getJavaType( String nomTipus ) {
		try {
			if( "BINARY".equals( nomTipus ) || "VARBINARY".equals( nomTipus ) || "TINYBLOB".equals( nomTipus ) 
					|| "MEDIUMBLOB".equals( nomTipus ) || "LONGBLOB".equals( nomTipus ) ){
				return GeneratorConstants.BYTE_ARRAY_JAVA_TYPE;
			} else if( "SMALLINT".equals( nomTipus ) || "MEDIUMINT".equals ( nomTipus ) 
					|| "INT".equals( nomTipus ) || "INTEGER".equals( nomTipus ) ){
				return GeneratorConstants.INTEGER_JAVA_TYPE;
			} else if( "INTEGER UNSIGNED".equals( nomTipus ) || "BIGINT".equals( nomTipus ) ){
				return GeneratorConstants.LONG_JAVA_TYPE;
			} else if( "FLOAT".equals( nomTipus ) ){
				return GeneratorConstants.FLOAT_JAVA_TYPE;
			} else if( "DOUBLE".equals( nomTipus ) ){
				return GeneratorConstants.DOUBLE_JAVA_TYPE;
			} else if( "DATETIME".equals( nomTipus ) || "TIMESTAMP".equals( nomTipus ) ){
				return GeneratorConstants.TIMESTAMP_JAVA_TYPE;
			} else if( "DECIMAL".equals( nomTipus ) || "TIMESTAMP".equals( nomTipus ) ){
				return GeneratorConstants.DECIMAL_JAVA_TYPE;
			} else if( "BIT".equals( nomTipus ) || "TINYINT".equals( nomTipus ) ){
				return GeneratorConstants.BOOLEAN_JAVA_TYPE;
			} else { return GeneratorConstants.STRING_JAVA_TYPE; }	
		}catch(Exception e){
			return GeneratorConstants.STRING_JAVA_TYPE;
		}
	}
	
	/**
	 * Escapem l'string donat
	 * @param valorAsObj
	 * @return
	 */
	public static String escapeString(String valorAsObj) {
		Matcher matcher = sqlTokenPattern.matcher( valorAsObj );
		StringBuffer sb = new StringBuffer();
		while( matcher.find() ){
			matcher.appendReplacement( sb, sqlTokens.get( matcher.group( 1 ) ) );
		}
		matcher.appendTail( sb );
		return sb.toString();
	}
	
	/**
	 * Transformem el resulSet a un llistat d'objectes
	 * @param rs
	 * @return
	 */
	public static List<Object> transformResultSetToObjectList(ResultSet rs, Object obj) throws DBException {
		Class<?> classe = obj.getClass();
		ResultSetUtils<Object> rsUtils = new ResultSetUtils<Object>();
		return rsUtils.transformResultSetToObjList( rs, classe );
	}
	
	/**
	 * 
	 * @param st
	 * @return
	 */
	public static String formatString(String st) {
	    String stString = SQLConstants.NULL_STRING;
	    if (st != null) {
	    	if (st.indexOf( SQLConstants.COMILLA_SIMPLE ) != -1){
	    		st = st.replaceAll( SQLConstants.COMILLA_SIMPLE, SQLConstants.DOBLE_COMILLA_SIMPLE );
	    	}
	    	stString = SQLConstants.COMILLA_SIMPLE.concat( st).concat( SQLConstants.COMILLA_SIMPLE );
	    }
	    return stString;
	}
	
	// Mètodes privats
	
	/**
	 * 
	 * @param objValue
	 * @param javaType
	 * @return
	 * @throws DBException
	 */
	private static String getResultValue( Object objValue, ColumnDBVO columnVO) throws DBException{
		String out = "";
		String javaType = getJavaType( columnVO.getType() );
		
		if( !objValue.getClass().getName().equalsIgnoreCase( javaType ) ){
			throw new DBException( ErrorConstants.ERROR_TYPE_MAPPING_CODI, "Els tipus de la columna " + columnVO.getName() + " no coincideixen. " +
					"S'ha trobat [" + objValue.getClass().getName() +"] quan s'esperava [" + javaType + "].");
		}
			
		if ( objValue instanceof java.lang.String ) {
			out = escapeString( (String) objValue );
			out = formatString( out );
		}else if( objValue instanceof java.sql.Timestamp ){
			out = "'" + objValue + "'";
		}else{
			out = "" + objValue;
		}
		return out;
	}
	
	/**
	 * Afegim la clàusula WHERE per cercar per la primary key de l'objecte
	 * @param registerVO
	 * @return
	 * @throws DBException
	 */
	private static String addClauseWherePK (RegisterDBVO registerVO) throws DBException {
		StringBuffer sb = new StringBuffer( SQLConstants.WHERE );
		try {
			ColumnDBVO columnVO = registerVO.getColumnPrimaryKey();
			// Check si es PK
			sb.append( columnVO.getName() ).append( SQLConstants.EQUALS );
			
			Method method = registerVO.getRegister().getClass().getMethod( columnVO.getMethod(), null );
			Object objValue = (Object) method.invoke( registerVO.getRegister(), null );
						
			String result = getResultValue( objValue, columnVO );
				
			sb.append(result);
			return sb.toString();	
		}catch(IllegalArgumentException e){
			throw new DBException( ErrorConstants.ERROR_ARGUMENT_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(IllegalAccessException e){
			throw new DBException( ErrorConstants.ERROR_ACCES_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(InvocationTargetException e){
			throw new DBException( ErrorConstants.ERROR_INVOCATION_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(SecurityException e){
			throw new DBException( ErrorConstants.ERROR_SECURITY_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}catch(NoSuchMethodException e){
			throw new DBException( ErrorConstants.ERROR_METHOD_MAPPING_CODI, ErrorConstants.ERROR_MAPPING_MSG + e.getMessage(), e );
		}
	}	
}