package edu.uoc.pfc.lcamps.webapp.catwings.action;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;

import edu.uoc.pfc.lcamps.webapp.catwings.constants.ActionConstants;
import edu.uoc.pfc.lcamps.webapp.catwings.constants.CatWingsConstants;
import edu.uoc.pfc.lcamps.webapp.catwings.form.InformacioPassatgersForm;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.AdultVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ContacteVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.InfantVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.NenVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.PagamentVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.VolResultVO;
import edu.uoc.pfc.lcamps.webapp.catwings.service.ReservaService;
import edu.uoc.pfc.lcamps.webapp.catwings.service.VolService;
import edu.uoc.pfc.lcamps.webapp.catwings.spring.CatWingsContext;

/**
 * Classe Action que ens servirà per crear una reserva
 * @author Llorenç Camps 
 */
public class CrearReservaAction extends GlobalAction{

	private static final Logger log = Logger.getLogger( CrearReservaAction.class );

	@SuppressWarnings("deprecation")
	@Override
	protected ActionForward doExecute(ActionMapping mapping, ActionForm form,
			HttpServletRequest req, HttpServletResponse res) throws Exception {
		
		ActionErrors errors = new ActionErrors();
		try{
	        log.debug( "CrearReservaAction() - Inici" );
	        InformacioPassatgersForm InformacioPassatgersForm = (InformacioPassatgersForm) form;
	        ConsultaVO consultaVO = transformInformacioPassatgersFormToConsultaVO( InformacioPassatgersForm );
	        // Generem la reserva
	        ReservaService reservaService = CatWingsContext.getInstance().getReservaService();
	        String numReserva = reservaService.createReserva( consultaVO );
	        // Insertem el numero de reserva generat en el formulari per mostrar-lo a l'usuari
	        InformacioPassatgersForm.setNumReserva( numReserva );
	        VolService volService = CatWingsContext.getInstance().getVolService();
	        VolResultVO result = volService.getInfoVols( consultaVO );
	        // Afegim el resultat a la request
	        req.setAttribute( ActionConstants.HTTP_REQUEST_ATTRIBUTE_RESULT_CONSULTA_VOLS, result );
	        return mapping.findForward( ActionConstants.MAPPING_OK );
		} catch (Exception e) {
			log.error( "CrearReservaAction() - Error:" + e.getMessage(), e );
			errors.add("CatWingsError",new ActionMessage("errors.actualitzar.places"));
			return mapping.findForward( ActionConstants.MAPPING_ERROR );
		}finally{
			if (!errors.isEmpty()) saveErrors(req,errors);
		}
	}
	
	/**
	 * A partir del formulari omplenarem el pojo amb les dades
	 * @param form
	 * @return
	 */
	private ConsultaVO transformInformacioPassatgersFormToConsultaVO( InformacioPassatgersForm form ){
		ConsultaVO consultaVO = new ConsultaVO();
		
		consultaVO.setTipusViatge( form.getTipusViatge() );
		consultaVO.setAeroportSortida( form.getOrigen() );
		consultaVO.setAeroportArribada( form.getDesti() );
		consultaVO.setDataSortida( form.getDataSortida() );
		consultaVO.setDataTornada( form.getDataTornada() );
		consultaVO.setNumAdults( form.getAdults() );
		consultaVO.setNumNens( form.getNens() );
		consultaVO.setNumInfants( form.getInfants() );
		consultaVO.setNumVolAnada( form.getSvolan() );
		if( form.getTipusViatge() == CatWingsConstants.VIATGE_ANADA_TORNADA ){
			consultaVO.setNumVolTornada( form.getSvoltor() );
		}
		consultaVO.setPreuFinal( form.getPreuFinal() );
		
		// Afegim la informació dels adults
		consultaVO.setAdults( addInfoAdults( form ) );
		// Afegim la informació dels nens
		consultaVO.setNens( addInfoNens( form ) );
		// Afegim la informació dels infants
		consultaVO.setInfants( addInfoInfants( form ) );
		// Afegim la informació del contacte
		consultaVO.setContacte( addInfoContacte( form ) );
		// Afegim la informació del pagament
		consultaVO.setPagament( addInfoPagament( form ) );
		
		return consultaVO;
	}

	/**
	 * Afegim la informació del pagament
	 * @param form
	 * @return
	 */
	private PagamentVO addInfoPagament(InformacioPassatgersForm form) {
		PagamentVO out = new PagamentVO();
		
		out.setTipoPagamentTarjeta( form.getTipopagamenttarjeta() );
		out.setNomTitularTarjeta( form.getNomtitulartarjeta() );
		out.setNumTarjeta( form.getNumtarjeta() );
		out.setCvvTarjeta( form.getCvvtarjeta() );
		out.setMesExpiracio( form.getMesexpiracio() );
		out.setAnyExpiracio( form.getAnyexpiracio() );
		
		return out;
	}

	/**
	 * Afegim la informació dels infants
	 * @param form
	 * @return
	 */
	private List<InfantVO> addInfoInfants(InformacioPassatgersForm form) {
		List<InfantVO> listOut = new ArrayList<InfantVO>();
		
		for( int i=1; i < form.getInfants()+1; i++ ){
		
			if( i == 1 ){
				InfantVO infant1 = new InfantVO();
				infant1.setNom( form.getNominfant1() );
				infant1.setCognoms( form.getCognomsinfant1() );
				infant1.setDataNaixement( form.getDianaixinfant1() + "/" + form.getMesnaixinfant1() + "/" + form.getAnynaixinfant1() );
				infant1.setViatgeAmb( form.getViatgeamb1() );
				listOut.add( infant1 );
			}
			
			if( i == 2 ){
				InfantVO infant2 = new InfantVO();
				infant2.setNom( form.getNominfant2() );
				infant2.setCognoms( form.getCognomsinfant2() );
				infant2.setDataNaixement( form.getDianaixinfant2() + "/" + form.getMesnaixinfant2() + "/" + form.getAnynaixinfant2() );
				infant2.setViatgeAmb( form.getViatgeamb2() );
				listOut.add( infant2 );
			}
			
			if( i == 3 ){
				InfantVO infant3 = new InfantVO();
				infant3.setNom( form.getNominfant3() );
				infant3.setCognoms( form.getCognomsinfant3() );
				infant3.setDataNaixement( form.getDianaixinfant3() + "/" + form.getMesnaixinfant3() + "/" + form.getAnynaixinfant3() );
				infant3.setViatgeAmb( form.getViatgeamb3() );
				listOut.add( infant3 );
			}
		}
		return listOut;
	}

	/**
	 * Afegim la informació dels nens
	 * @param form
	 * @return
	 */
	private List<NenVO> addInfoNens(InformacioPassatgersForm form) {
		List<NenVO> listOut = new ArrayList<NenVO>();
		
		for( int i=1; i < form.getNens()+1; i++ ){
			
			if( i == 1 ){
				NenVO nen1 = new NenVO();
				nen1.setNom( form.getNomnen1() );
				nen1.setCognoms( form.getCognomsnen1() );
				nen1.setDataNaixement( form.getDianaixnen1() + "/" + form.getMesnaixnen1() + "/" + form.getAnynaixnen1() );
				listOut.add( nen1 );
			}
			
			if( i == 2 ){
				NenVO nen2 = new NenVO();
				nen2.setNom( form.getNomnen2() );
				nen2.setCognoms( form.getCognomsnen2() );
				nen2.setDataNaixement( form.getDianaixnen2() + "/" + form.getMesnaixnen2() + "/" + form.getAnynaixnen2() );
				listOut.add( nen2 );
			}
			
			if( i == 3 ){
				NenVO nen3 = new NenVO();
				nen3.setNom( form.getNomnen3() );
				nen3.setCognoms( form.getCognomsnen3() );
				nen3.setDataNaixement( form.getDianaixnen3() + "/" + form.getMesnaixnen3() + "/" + form.getAnynaixnen3() );
				listOut.add( nen3 );
			}
			
			if( i == 4 ){
				NenVO nen4 = new NenVO();
				nen4.setNom( form.getNomnen4() );
				nen4.setCognoms( form.getCognomsnen4() );
				nen4.setDataNaixement( form.getDianaixnen4() + "/" + form.getMesnaixnen4() + "/" + form.getAnynaixnen4() );
				listOut.add( nen4 );
			}
			
			if( i == 5 ){
				NenVO nen5 = new NenVO();
				nen5.setNom( form.getNomnen5() );
				nen5.setCognoms( form.getCognomsnen5() );
				nen5.setDataNaixement( form.getDianaixnen5() + "/" + form.getMesnaixnen5() + "/" + form.getAnynaixnen5() );
				listOut.add( nen5 );
			}
		}
		
		return listOut;
	}

	/**
	 * Afegim la informació dels adults
	 * @param form
	 * @return
	 */
	private List<AdultVO> addInfoAdults(InformacioPassatgersForm form) {
		List<AdultVO> listOut = new ArrayList<AdultVO>();
		
		for( int i=1; i < form.getAdults()+1; i++ ){
			
			if( i == 1 ){
				AdultVO adult1 = new AdultVO();
				adult1.setTitol( form.getTitoladult1() );
				adult1.setNom( form.getNomadult1() );
				adult1.setCognoms( form.getCognomsadult1() );
				adult1.setDataNaixement( form.getDianaixadult1() + "/" + form.getMesnaixadult1() + "/" + form.getAnynaixadult1() );
				listOut.add( adult1 );
			}
			
			if( i == 2 ){
				AdultVO adult2 = new AdultVO();
				adult2.setTitol( form.getTitoladult2() );
				adult2.setNom( form.getNomadult2() );
				adult2.setCognoms( form.getCognomsadult2() );
				adult2.setDataNaixement( form.getDianaixadult2() + "/" + form.getMesnaixadult2() + "/" + form.getAnynaixadult2() );
				listOut.add( adult2 );
			}
			
			if( i == 3 ){
				AdultVO adult3 = new AdultVO();
				adult3.setTitol( form.getTitoladult3() );
				adult3.setNom( form.getNomadult3() );
				adult3.setCognoms( form.getCognomsadult3() );
				adult3.setDataNaixement( form.getDianaixadult3() + "/" + form.getMesnaixadult3() + "/" + form.getAnynaixadult3() );
				listOut.add( adult3 );
			}
			
			if( i == 4 ){
				AdultVO adult4 = new AdultVO();
				adult4.setTitol( form.getTitoladult4() );
				adult4.setNom( form.getNomadult4() );
				adult4.setCognoms( form.getCognomsadult4() );
				adult4.setDataNaixement( form.getDianaixadult4() + "/" + form.getMesnaixadult4() + "/" + form.getAnynaixadult4() );
				listOut.add( adult4 );
			}
			
			if( i == 5 ){
				AdultVO adult5 = new AdultVO();
				adult5.setTitol( form.getTitoladult5() );
				adult5.setNom( form.getNomadult5() );
				adult5.setCognoms( form.getCognomsadult5() );
				adult5.setDataNaixement( form.getDianaixadult5() + "/" + form.getMesnaixadult5() + "/" + form.getAnynaixadult5() );
				listOut.add( adult5 );
			}
		}
		
		return listOut;
	}

	/**
	 * Afegim la info del contacte
	 * @param form
	 * @return
	 */
	private ContacteVO addInfoContacte(InformacioPassatgersForm form) {
		ContacteVO out = new ContacteVO();
		
		out.setTitol( form.getTitolcontacte() );
		out.setNom( form.getNomcontacte() );
		out.setCognoms( form.getCognomscontacte() );
		out.setAdreca( form.getAdrecacontacte() );
		out.setPoblacio( form.getPoblaciocontacte() );
		out.setProvincia( form.getProvinciacontacte() );
		out.setCodiPostal( form.getCpcontacte() );
		out.setPais( form.getPaiscontacte() );
		out.setTelf( form.getTelfcontacte() );
		out.setTelf2( form.getTelfcontacte2() );
		out.setEmail( form.getEmailcontacte() );
		
		return out;
	}
}
