package edu.uoc.pfc.lcamps.webapp.catwings.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import edu.uoc.pfc.lcamps.webapp.catwings.constants.ActionConstants;
import edu.uoc.pfc.lcamps.webapp.catwings.constants.CatWingsConstants;
import edu.uoc.pfc.lcamps.webapp.catwings.form.InformacioPassatgersForm;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.VolResultVO;
import edu.uoc.pfc.lcamps.webapp.catwings.service.VolService;
import edu.uoc.pfc.lcamps.webapp.catwings.spring.CatWingsContext;


/**
 * Classe Action que ens servirà per obtenir les dades dels vols
 * @author Llorenç Camps 
 */
public class DadesPagamentAction extends GlobalAction{

	private static final Logger log = Logger.getLogger( DadesPagamentAction.class );

	@Override
	protected ActionForward doExecute(ActionMapping mapping, ActionForm form,
			HttpServletRequest req, HttpServletResponse res) throws Exception {
		
		try{
	        log.debug( "DadesPagamentAction() - Inici" );
	        InformacioPassatgersForm seleccioVolForm = (InformacioPassatgersForm) form;
	        VolService volService = CatWingsContext.getInstance().getVolService();
	        VolResultVO result = volService.getInfoVols( transformInformacioPassatgersFormToConsultaVO( seleccioVolForm ) );
	        // Afegim el resultat a la request
	        req.setAttribute( ActionConstants.HTTP_REQUEST_ATTRIBUTE_RESULT_CONSULTA_VOLS, result );
	        return mapping.findForward( ActionConstants.MAPPING_OK );
		} catch (Exception e) {
			log.error( "DadesPagamentAction() - Error:" + e.getMessage(), e );
			req.setAttribute( ActionConstants.HTTP_REQUEST_ATTRIBUTE_RESULT_CONSULTA_VOLS, new VolResultVO() );
			return mapping.findForward( ActionConstants.MAPPING_OK );
		}
	}
	
	/**
	 * A partir del formulari omplenarem el pojo amb les dades
	 * @param form
	 * @return
	 */
	private ConsultaVO transformInformacioPassatgersFormToConsultaVO( InformacioPassatgersForm form ){
		ConsultaVO consultaVO = new ConsultaVO();
		
		consultaVO.setTipusViatge( form.getTipusViatge() );
		consultaVO.setAeroportSortida( form.getOrigen() );
		consultaVO.setAeroportArribada( form.getDesti() );
		consultaVO.setDataSortida( form.getDataSortida() );
		consultaVO.setDataTornada( form.getDataTornada() );
		consultaVO.setNumAdults( form.getAdults() );
		consultaVO.setNumNens( form.getNens() );
		consultaVO.setNumInfants( form.getInfants() );
		consultaVO.setNumVolAnada( form.getSvolan() );
		if( form.getTipusViatge() == CatWingsConstants.VIATGE_ANADA_TORNADA ){
			consultaVO.setNumVolTornada( form.getSvoltor() );
		}
		
		return consultaVO;
	}
}
