package edu.uoc.pfc.lcamps.webapp.catwings.bd.dao;

import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO;

/**
 * Interfície que ens servirà per la gestió del DAO del passatger
 * @author Llorenç Camps 
 */
public interface PassatgerDAO {
	
	/**
	 * insertem el passatger i retorna l'id generat
	 * @param sessio
	 * @param passatger
	 * @return
	 * @throws DBException
	 */
	int insertPassatger( SessionFW sessio, PassatgerVO passatger ) throws DBException;
	
	/**
	 * Obtenim el contacte a partir del codi i de l'email
	 * @param codi
	 * @param email
	 * @return
	 * @throws DBException
	 */
	PassatgerVO selectContacte(String codi, String email) throws DBException;
	
	/**
	 * Obtenim el contacte a partir del codi
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	PassatgerVO selectContacte(String codi) throws DBException;
	
	/**
	 * Obtenim el llistat d'adults a partir del codi
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> selectAdult(String codi) throws DBException;
	
	/**
	 * Obtenim el llistat de nens a partir del codi
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> selectNen(String codi) throws DBException;
	
	/**
	 * Obtenim el llistat d'infants a partir del codi
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> selectInfant(String codi) throws DBException;
	
	/**
	 * Actualitza el passatger
	 * @param sessio
	 * @param obj
	 * @throws DBException
	 */
	void updatePassatger(SessionFW sessio, PassatgerVO obj) throws DBException; 
		
	/**
	 * Esborrem el passatger
	 * @param sessio
	 * @param obj
	 * @return
	 * @throws DBException
	 */
	void deletePassatger(SessionFW sessio, PassatgerVO obj) throws DBException;
	
	/**
	 * Obtenim tots els passatgers de la reserva
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> selectPassatgers(String codi) throws DBException;
	
	/**
	 * Obtenim tots els passatgers de la reserva que paguen (tots - infants)
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> selectPassatgersPaguen(String codi) throws DBException;
	
}
