package edu.uoc.pfc.lcamps.webapp.catwings.bd.dao;

import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO;

/**
 * Interfície que ens servirà per la gestió del DAO de l'aeroport
 * @author Llorenç Camps 
 */
public interface VolDAO {
	
	/**
	 * obtenim el llistat de vols segons la data de sortida, origen, desti i les places
	 * @param origen
	 * @param desti
	 * @param dataSortida
	 * @param numPlaces
	 * @return
	 * @throws DBException
	 */
	List<VolVO> selectVolPerData( String origen, String desti, String dataSortida, int numPlaces ) throws DBException;
	
	/**
	 * Obtenim el vol a partir del número de vol
	 * @param numvol
	 * @return
	 * @throws DBException
	 */
	VolVO selectVol(int numvol) throws DBException;
	
	/**
	 * Obtenim el número de places disponibles en el vol a partir del seu número
	 * @param numVol
	 * @return
	 * @throws DBException
	 */
	int selectNumPlacesVol(int numVol) throws DBException;
	
	/**
	 * Actualitzem la informació del vol
	 * @param vol
	 * @return
	 * @throws DBException
	 */
	void updateVol(SessionFW sessio, VolVO vol) throws DBException;

	/**
	 * Esobrrem el vol
	 * @param numvol
	 * @return
	 * @throws DBException
	 */
	void deleteVol(SessionFW sessio, VolVO vol) throws DBException; 
	
	/**
	 * Obtenim el preu d'un vol a partir del seu número
	 * @param numVol
	 * @return
	 * @throws DBException
	 */
	int selectPreuVol(int numVol) throws DBException; 
}
