package edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.impl;

import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AbstractDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AeroportDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.AeroportVO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.sql.AeroportQueryProvider;

/**
 * Classe que implementa la interfície del DAO de l'aeroport
 * @author Llorenç Camps 
 */
public class AeroportDAOImpl extends AbstractDAO<AeroportVO> implements AeroportDAO{

	private AeroportQueryProvider queryProvider;

	/**
	 * @param queryProvider the queryProvider to set
	 */
	public void setQueryProvider(AeroportQueryProvider queryProvider) {
		this.queryProvider = queryProvider;
	}

	/**
	 * @return the queryProvider
	 */
	public AeroportQueryProvider getQueryProvider() {
		return queryProvider;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AeroportDAO#selectAeroport(java.lang.String)
	 */
	@Override
	public AeroportVO selectAeroport(String codiiata) throws DBException {
		String sql = queryProvider.getQueryAeroportByCodiIata( codiiata );
		List<AeroportVO> aeroport = super.findList( sql, new AeroportVO() );
		if( aeroport.size() > 0 ) return aeroport.get( 0 );
		else return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AeroportDAO#insertAeroport(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.AeroportVO)
	 */
	@Override
	public int insertAeroport(SessionFW sessio, AeroportVO obj)
			throws DBException {
		return super.insert(sessio, obj);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AeroportDAO#updateAeroport(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.AeroportVO)
	 */
	@Override
	public void updateAeroport(SessionFW sessio, AeroportVO obj)
			throws DBException {
		super.update(sessio, obj);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AeroportDAO#deleteAeroport(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.AeroportVO)
	 */
	@Override
	public void deleteAeroport(SessionFW sessio, AeroportVO obj) throws DBException {
		super.delete(sessio, obj);
	}

}
