package edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.impl;

import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AbstractDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.sql.PassatgerQueryProvider;

/**
 * Classe que implementa la interfície del DAO del passatger
 * @author Llorenç Camps 
 */
public class PassatgerDAOImpl extends AbstractDAO<PassatgerVO> implements PassatgerDAO {

	private PassatgerQueryProvider queryProvider;

	/**
	 * @param queryProvider the queryProvider to set
	 */
	public void setQueryProvider(PassatgerQueryProvider queryProvider) {
		this.queryProvider = queryProvider;
	}

	/**
	 * @return the queryProvider
	 */
	public PassatgerQueryProvider getQueryProvider() {
		return queryProvider;
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#insertPassatger(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO)
	 */
	@Override
	public int insertPassatger(SessionFW sessio, PassatgerVO obj)
			throws DBException {
		return super.insert( sessio, obj );
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#selectContacte(java.lang.String, java.lang.String)
	 */
	@Override
	public PassatgerVO selectContacte(String codi, String email) throws DBException{
		String sql = queryProvider.getQuerySelectContacteByEmailCodi(codi, email);
		List<PassatgerVO> passatgers = super.findList(sql, new PassatgerVO());
		if( passatgers.size() > 0 ) return passatgers.get( 0 );
		else return null;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#selectContacte(java.lang.String)
	 */
	@Override
	public PassatgerVO selectContacte(String codi) throws DBException {
		String sql = queryProvider.getQuerySelectContacteByCodi(codi);
		List<PassatgerVO> passatgers = super.findList(sql, new PassatgerVO());
		if( passatgers.size() > 0 ) return passatgers.get( 0 );
		else return null;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#selectAdult(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> selectAdult(String codi) throws DBException {
		String sql = queryProvider.getQuerySelectAdultsByCodi(codi);
		return super.findList(sql, new PassatgerVO());
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#selectNen(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> selectNen(String codi) throws DBException {
		String sql = queryProvider.getQuerySelectNensByCodi(codi);
		return super.findList(sql, new PassatgerVO());
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#selectInfant(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> selectInfant(String codi) throws DBException {
		String sql = queryProvider.getQuerySelectInfantsByCodi(codi);
		return super.findList(sql, new PassatgerVO());
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AbstractDAO#updatePassatger(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, java.lang.Object)
	 */
	@Override
	public void updatePassatger(SessionFW sessio, PassatgerVO obj) throws DBException {
		super.update(sessio, obj);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#deletePassatger(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO)
	 */
	@Override
	public void deletePassatger(SessionFW sessio, PassatgerVO obj) throws DBException {
		super.delete(sessio, obj);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#selectPassatgers(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> selectPassatgers(String codi) throws DBException {
		String sql = queryProvider.getQuerySelectAllPassatgersByCodi(codi);
		return super.findList(sql, new PassatgerVO());
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO#selectPassatgersPaguen(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> selectPassatgersPaguen(String codi)
			throws DBException {
		String sql = queryProvider.getQuerySelectAllPassatgersPaguenByCodi( codi );
		return super.findList(sql, new PassatgerVO());
	}
}
