package edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.impl;

import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AbstractDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.ReservaDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.ReservaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.sql.ReservaQueryProvider;

/**
 * Classe que implementa la interfície del DAO de la reserva
 * @author Llorenç Camps 
 */
public class ReservaDAOImpl extends AbstractDAO<ReservaVO> implements ReservaDAO {

	private ReservaQueryProvider queryProvider;

	/**
	 * @param queryProvider the queryProvider to set
	 */
	public void setQueryProvider(ReservaQueryProvider queryProvider) {
		this.queryProvider = queryProvider;
	}

	/**
	 * @return the queryProvider
	 */
	public ReservaQueryProvider getQueryProvider() {
		return queryProvider;
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.ReservaDAO#openTransaccio(boolean)
	 */
	@Override
	public SessionFW openTransaccio(boolean autocommit) throws DBException {
		return super.openTransaction( autocommit );
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.ReservaDAO#closeTransaccio(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW)
	 */
	@Override
	public void closeTransaccio(SessionFW sessio) throws DBException {
		super.closeTransaction( sessio );
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.ReservaDAO#insertReserva(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.ReservaVO)
	 */
	@Override
	public int insertReserva(SessionFW sessio, ReservaVO obj) throws DBException {
		return super.insert( sessio, obj );		
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.ReservaDAO#deleteReserva(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.ReservaVO)
	 */
	@Override
	public void deleteReserva(SessionFW sessio, ReservaVO obj) throws DBException {
		super.delete( sessio, obj );
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.ReservaDAO#selectReserva(java.lang.String)
	 */
	@Override
	public ReservaVO selectReserva(String codi) throws DBException {
		String sql = queryProvider.getQuerySelectReservaByCodiReserva( codi );
		List<ReservaVO> reserves = super.findList( sql, new ReservaVO() );
		if( reserves.size() > 0 ) return reserves.get( 0 );
		else return null;
	}
}
