package edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.impl;

import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.AbstractDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.VolDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.sql.VolQueryProvider;

/**
 * Classe que implementa la interfície del DAO del vol
 * @author Llorenç Camps 
 */
public class VolDAOImpl extends AbstractDAO<VolVO> implements VolDAO {
	
	private VolQueryProvider queryProvider;

	/**
	 * @param queryProvider the queryProvider to set
	 */
	public void setQueryProvider(VolQueryProvider queryProvider) {
		this.queryProvider = queryProvider;
	}

	/**
	 * @return the queryProvider
	 */
	public VolQueryProvider getQueryProvider() {
		return queryProvider;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.VolDAO#selectVolPerData(java.lang.String, java.lang.String, java.lang.String, int)
	 */
	@Override
	public List<VolVO> selectVolPerData(String origen, String desti, String dataSortida, int numPlaces) throws DBException {
		String sql = queryProvider.getQuerySelectVolByInfo( origen, desti, dataSortida, numPlaces );
		return super.findList( sql, new VolVO() );
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.VolDAO#selectVol(int)
	 */
	@Override
	public VolVO selectVol(int numVol) throws DBException {
		String sql = queryProvider.getQuerySelectVolByNumVol( numVol );
		List<VolVO> vols = super.findList( sql, new VolVO() );
		if( vols.size() > 0 ) return vols.get( 0 );
		else return null;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.VolDAO#selectNumPlacesVol(int)
	 */
	@Override
	public int selectNumPlacesVol(int numVol) throws DBException {
		String sql = queryProvider.getQuerySelectVolByNumVol( numVol );
		List<VolVO> vols = super.findList( sql, new VolVO() );
		if( vols.size() > 0 ) return vols.get( 0 ).getNumPlaces(); 
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.VolDAO#updateVol(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO)
	 */
	@Override
	public void updateVol( SessionFW sessio, VolVO obj)	throws DBException {
		super.update(sessio, obj);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.VolDAO#deleteVol(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO)
	 */
	@Override
	public void deleteVol( SessionFW sessio, VolVO obj) throws DBException {
		super.delete(sessio, obj);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.VolDAO#selectPreuVol(int)
	 */
	@Override
	public int selectPreuVol(int numVol) throws DBException {
		String sql = queryProvider.getQuerySelectVolByNumVol( numVol );
		List<VolVO> vols = super.findList( sql, new VolVO() );
		if( vols.size() > 0 ) return vols.get( 0 ).getPreuVol();
		return 0;
	}
}
