package edu.uoc.pfc.lcamps.webapp.catwings.service;

import java.util.List;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO;

/**
 * Interfície que ens serveix per gestionar les diferents operacions sobre el passatger
 * @author Llorenç Camps 
 */
public interface PassatgerService {
	
	/**
	 * insertem el passatger i retorna l'id generat
	 * @param sessio
	 * @param passatger
	 * @return
	 * @throws DBException
	 */
	int crearPassatger( SessionFW sessio, PassatgerVO passatger) throws DBException;
	
	/**
	 * obtenim el contacte a partir del correu i el codi
	 * @param codi
	 * @param email
	 * @return
	 * @throws DBException
	 */
	PassatgerVO getContacteByEmailCodi( String codi, String email ) throws DBException;
	
	/**
	 * obtenim el contacte a partir del codi
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	PassatgerVO getContacteByCodi( String codi ) throws DBException;
	
	/**
	 * Obtenim el llistat de tots els passatgers
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> getAllPassatgers( String codi ) throws DBException;
	
	/**
	 * Obtenim el llistat de tots els passatgers que paguen
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> getAllPassatgersPaguen( String codi ) throws DBException;
	
	/**
	 * Esborrem el passatger
	 * @param sessio
	 * @param passatger
	 * @throws DBException
	 */
	void deletePassatger( SessionFW sessio, PassatgerVO passatger) throws DBException;
	
	/**
	 * obtenim el llistat d'adults a partir del codi
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> getAdultsByCodi( String codi ) throws DBException;
	
	/**
	 * obtenim el llistat de nens a partir del codi
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> getNensByCodi( String codi ) throws DBException;
	
	/**
	 * obtenim el llistat d'infants a partir del codi
	 * @param codi
	 * @return
	 * @throws DBException
	 */
	List<PassatgerVO> getInfantsByCodi( String codi ) throws DBException;
}
