package edu.uoc.pfc.lcamps.webapp.catwings.service;

import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.ReservaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ReservaInfoVO;

/**
 * Interfície que ens serveix per gestionar les diferents operacions sobre la reserva
 * @author Llorenç Camps 
 */
public interface ReservaService {
	
	/**
	 * Donada la informació que l'usuari ha anat introduït amb totes les dades
	 * es crea una reserva i es retorna el número de reserva
	 * @param consultaVO
	 * @return el número de reserva
	 */
	String createReserva (ConsultaVO consultaVO ) throws Exception;
	
	/**
	 * Donat el codi de reserva i email validem si les dades que ha introduit l'usuari son correctes 
	 * i en cas afirmatiu es retorna les dades de la reserva
	 * @param codiReserva
	 * @param email
	 * @return
	 * @throws Exception
	 */
	ReservaVO getReserva( String codiReserva, String email ) throws Exception;
	
	/**
	 * Donada la reserva obtenim tota la seva informació de passatgers / vols / contacte per mostrar
	 * @param reserva
	 * @return
	 * @throws Exception
	 */
	ReservaInfoVO getInfoReserva( ReservaVO reserva ) throws Exception;
	
	/**
	 * Donat el codi de reserva retornem l'import total a retornar a l'usuari en cas de cancelar-la,
	 * L'import total és el preu que ha pagat l'usuari menys un 30%
	 * @param codiReserva
	 * @return
	 * @throws Exception
	 */
	double getPreuReservaRetornar( String codiReserva ) throws Exception;
	
	/**
	 * Donat el codi de reserva, eliminem la reserva, els passatgers i actualitzem el número de places de vol
	 * @param codiReserva
	 * @throws Exception
	 */
	void deleteReserva( String codiReserva ) throws Exception;
	
}
