package edu.uoc.pfc.lcamps.webapp.catwings.service;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.VolResultVO;

/**
 * Interfície que ens serveix per gestionar les diferents operacions sobre el vol
 * @author Llorenç Camps 
 */
public interface VolService {
	
	/**
	 * A partir de les dades que ha introdït l'usuari buscarem el llistat de vols
	 * segons els paràmetres passats
	 * @param consultaVO
	 * @return
	 */
	VolResultVO getInfoLlistatVols( ConsultaVO consultaVO );
	
	/**
	 * A partir de les dades que ha introdït l'usuari buscarem la informacio dels vols
	 * segons els paràmetres passats
	 * @param consultaVO
	 * @return
	 */
	VolResultVO getInfoVols( ConsultaVO consultaVO );
	
	/**
	 * Obtenim el Vol
	 * @param numVol
	 * @return
	 * @throws DBException
	 */
	VolVO getVolByNumVol ( int numVol ) throws DBException;
	
	/**
	 * Obtenim el vol amb els noms de l'origen/desti extrets de les dades de l'aeroport
	 * @param numVol
	 * @return
	 * @throws DBException
	 */
	VolVO getVolUpdateNomAeroportByNumVol( int numVol ) throws DBException;
	
	/**
	 * A partir del numero de vol obtenim el número de places disponibles
	 * @param numVol
	 * @return
	 */
	int getNumPlacesDisponibles( int numVol );
	
	/**
	 * Actualitzem el vol
	 * @param sessio
	 * @param vol
	 * @throws DBException
	 */
	void updateVol( SessionFW sessio, VolVO vol ) throws DBException;
	
	/**
	 * Esborrem el vol
	 * @param sessio
	 * @param vol
	 * @throws DBException
	 */
	void deleteVol( SessionFW sessio, VolVO vol ) throws DBException;
}
