package edu.uoc.pfc.lcamps.webapp.catwings.service.impl;

import java.util.List;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.PassatgerDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO;
import edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService;

/**
 * Classe que implementa la interfície que gestiona el passatger
 * @author Llorenç Camps 
 */
public class PassatgerServiceImpl implements PassatgerService {
	
	private static final Logger log = Logger.getLogger( PassatgerServiceImpl.class );
	
	private PassatgerDAO passatgerDAO;

	/**
	 * @param passatgerDAO the passatgerDAO to set
	 */
	public void setPassatgerDAO(PassatgerDAO passatgerDAO) {
		this.passatgerDAO = passatgerDAO;
	}

	/**
	 * @return the passatgerDAO
	 */
	public PassatgerDAO getPassatgerDAO() {
		return passatgerDAO;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#crearPassatger(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO)
	 */
	@Override
	public int crearPassatger(SessionFW sessio, PassatgerVO passatger) throws DBException {
		log.debug( "crearPassatger() - init" ); 
		return passatgerDAO.insertPassatger(sessio, passatger);
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#getContacteByEmailCodi(java.lang.String, java.lang.String)
	 */
	@Override
	public PassatgerVO getContacteByEmailCodi( String codi, String email ) throws DBException{
		log.debug( "getContacteByEmailCodi() - init" );
		return passatgerDAO.selectContacte(codi, email);
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#getContacteByCodi(java.lang.String)
	 */
	@Override
	public PassatgerVO getContacteByCodi( String codi ) throws DBException{
		log.debug( "getContacteByCodi() - init" );
		return passatgerDAO.selectContacte(codi);
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#getAllPassatgers(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> getAllPassatgers( String codi ) throws DBException{
		log.debug( "getAllPassatgers() - init" );
		return passatgerDAO.selectPassatgers( codi );
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#getAllPassatgersPaguen(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> getAllPassatgersPaguen(String codi)
			throws DBException {
		log.debug( "getAllPassatgersPaguen() - init" );
		return passatgerDAO.selectPassatgersPaguen( codi );
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#deletePassatger(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO)
	 */
	@Override
	public void deletePassatger(SessionFW sessio, PassatgerVO passatger)
			throws DBException {
		log.debug( "deletePassatger() - init" ); 
		passatgerDAO.deletePassatger(sessio, passatger);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#getAdultsByCodi(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> getAdultsByCodi(String codi) throws DBException {
		log.debug( "getAdultsByCodi() - init" );
		return passatgerDAO.selectAdult(codi);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#getNensByCodi(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> getNensByCodi(String codi) throws DBException {
		log.debug( "getNensByCodi() - init" );
		return passatgerDAO.selectNen(codi);
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService#getInfantsByCodi(java.lang.String)
	 */
	@Override
	public List<PassatgerVO> getInfantsByCodi(String codi) throws DBException {
		log.debug( "getInfantsByCodi() - init" );
		return passatgerDAO.selectInfant(codi);
	}

}
