package edu.uoc.pfc.lcamps.webapp.catwings.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.ReservaDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.PassatgerVO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.ReservaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO;
import edu.uoc.pfc.lcamps.webapp.catwings.constants.CatWingsConstants;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.AdultVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ContacteVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.InfantVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.NenVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ReservaInfoVO;
import edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService;
import edu.uoc.pfc.lcamps.webapp.catwings.service.ReservaService;
import edu.uoc.pfc.lcamps.webapp.catwings.service.VolService;
import edu.uoc.pfc.lcamps.webapp.catwings.utils.CatwingsUtils;

/**
 * Classe que implementa la interfície que gestiona la reserva
 * @author Llorenç Camps 
 */
public class ReservaServiceImpl implements ReservaService {
	
	private static final Logger log = Logger.getLogger( ReservaServiceImpl.class );
	
	private ReservaDAO reservaDAO;
	
	private PassatgerService passatgerService;
	
	private VolService volService;

	/**
	 * @param reservaDAO the reservaDAO to set
	 */
	public void setReservaDAO(ReservaDAO reservaDAO) {
		this.reservaDAO = reservaDAO;
	}

	/**
	 * @return the reservaDAO
	 */
	public ReservaDAO getReservaDAO() {
		return reservaDAO;
	}
	
	/**
	 * @param volService the volService to set
	 */
	public void setVolService(VolService volService) {
		this.volService = volService;
	}

	/**
	 * @return the volService
	 */
	public VolService getVolService() {
		return volService;
	}

	/**
	 * @param passatgerService the passatgerService to set
	 */
	public void setPassatgerService(PassatgerService passatgerService) {
		this.passatgerService = passatgerService;
	}

	/**
	 * @return the passatgerService
	 */
	public PassatgerService getPassatgerService() {
		return passatgerService;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.ReservaService#createReserva(edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO)
	 */
	@Override
	public String createReserva(ConsultaVO consultaVO) throws Exception {
		log.debug( "createReserva() - Inici" );
		// Generem el codi de reserva
		String numReserva = CatwingsUtils.calculCodiReserva( consultaVO.getContacte().getNom(), consultaVO.getContacte().getCognoms(), consultaVO.getContacte().getProvincia() );
		
		// Abans de crear la reserva comprovem si hi han places disponibles
		int numPlaces = consultaVO.getNumAdults() + consultaVO.getNumNens();
		
		int placesVolAnada = volService.getNumPlacesDisponibles( consultaVO.getNumVolAnada() );
		if( placesVolAnada < numPlaces ) throw new Exception( "No hi han places disponibles" );
		
		if( consultaVO.getTipusViatge() == CatWingsConstants.VIATGE_ANADA_TORNADA ){
			int placesVolTornada = volService.getNumPlacesDisponibles( consultaVO.getNumVolTornada() );
			if( placesVolTornada < numPlaces ) throw new Exception( "No hi han places disponibles" );
		}
		SessionFW sessio = null;	
		try{
			// Comencem la transacció de crear la reserva
			sessio = reservaDAO.openTransaccio( false );
			
			// Modifiquem les places de vol
			VolVO volAnada = volService.getVolByNumVol( consultaVO.getNumVolAnada() );
			volAnada.setNumPlaces( volAnada.getNumPlaces() - numPlaces );
			//Actualitzem les places de vol
			volService.updateVol( sessio, volAnada );
			
			if( consultaVO.getTipusViatge() == CatWingsConstants.VIATGE_ANADA_TORNADA ){
				// Modifiquem les places de vol
				VolVO volTornada = volService.getVolByNumVol( consultaVO.getNumVolTornada() );
				volTornada.setNumPlaces( volTornada.getNumPlaces() - numPlaces );
				//Actualitzem les places de vol
				volService.updateVol( sessio, volTornada );
			}
			
			// Insertem les dades del contacte
			PassatgerVO contacte = createPassatgerContacte( consultaVO.getContacte(), numReserva );
			passatgerService.crearPassatger( sessio, contacte );
			
			// Insertem les dades dels passatgers
			for( AdultVO aux : consultaVO.getAdults() ){
				PassatgerVO passatgerAux = createPassatgerVOFromAdult( aux, numReserva );
				passatgerService.crearPassatger( sessio, passatgerAux );
			}
			
			// Insertem les dades dels nens
			for( NenVO aux : consultaVO.getNens() ){
				PassatgerVO passatgerAux = createPassatgerVOFromNen( aux, numReserva );
				passatgerService.crearPassatger( sessio, passatgerAux );
			}
			
			// Insertem les dades dels infants
			for( InfantVO aux : consultaVO.getInfants() ){
				PassatgerVO passatgerAux = createPassatgerVOFromInfant( aux, numReserva );
				passatgerService.crearPassatger( sessio, passatgerAux );
			}
			
			// Finalment insertem la reserva
			ReservaVO reserva = createReservaVO(numReserva, consultaVO);
			reservaDAO.insertReserva( sessio, reserva );
						
			// Si tot ha anat b fem el commit i guardem tots els canvis
			sessio.getTransactionFW().commit();
			
		}catch(Exception e){
			log.debug( "createReserva() - Error:" + e.getMessage() );
			// Fem rollback
			if( sessio != null )sessio.getTransactionFW().rollback();
			throw e;
		}finally{
			reservaDAO.closeTransaccio(sessio);
		}
		
		log.debug( "createReserva() - Fi [numReserva=" + numReserva + "]" );
		return numReserva;
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.ReservaService#getReserva(java.lang.String, java.lang.String)
	 */
	@Override
	public ReservaVO getReserva(String codiReserva, String email) throws Exception {
		log.debug( "getReserva() - Inici [codiReserva=" + codiReserva + "]" );
		PassatgerVO contacte = passatgerService.getContacteByEmailCodi(codiReserva, email);
		ReservaVO reserva = reservaDAO.selectReserva(codiReserva);
		
		//Si no trobo algun dels 2 enviem una excepcio pk no ha introduit les dades correctament
		if( contacte == null || reserva == null ) throw new Exception("Les dades introduides son incorrectes");
		
		log.debug( "getReserva() - Fi" );
		return reserva;
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.ReservaService#getInfoReserva(edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.ReservaVO)
	 */
	@Override
	public ReservaInfoVO getInfoReserva(ReservaVO reserva) throws Exception {
		log.debug( "getInfoReserva() - Inici" );
		ReservaInfoVO out = new ReservaInfoVO();
		
		// Insertem el codi de reserva
		out.setCodiReserva( reserva.getCodiReserva() );
		
		// Insertem el vol d'anada
		out.setVolAnada( volService.getVolUpdateNomAeroportByNumVol( reserva.getVolAnada() ) );
		
		if( null != reserva.getVolTornada() ){
			// Insertem el vol de tornada
			out.setVolTornada( volService.getVolUpdateNomAeroportByNumVol( reserva.getVolTornada() ) );
		}
		
		// Insertem la informació del contacte
		out.setContacte( createContacteVO( passatgerService.getContacteByCodi( reserva.getCodiReserva() ) ) );
		
		// Insertem els noms dels contactes
		out.setNomsPassatgers( createListNoms( passatgerService.getAllPassatgers( reserva.getCodiReserva() ) ) );
		
		log.debug( "getInfoReserva() - Fi" );
		return out;
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.ReservaService#getPreuReservaRetornar(java.lang.String)
	 */
	@Override
	public double getPreuReservaRetornar(String codiReserva) throws Exception {
		log.debug( "getPreuReservaRetornar() - Inici [codiReserva=" + codiReserva + "]" );
		ReservaVO reserva = reservaDAO.selectReserva(codiReserva);
		int preu  = 0;
		
		List<PassatgerVO> passatgers = passatgerService.getAllPassatgersPaguen( codiReserva );
		
		VolVO volAnada = volService.getVolByNumVol( reserva.getVolAnada() );
		
		preu = volAnada.getPreuVol() * passatgers.size();
		
		if( null != reserva.getVolTornada() ){
			VolVO volTornada = volService.getVolByNumVol( reserva.getVolTornada() );
			preu = preu + ( volTornada.getPreuVol() * passatgers.size() );
		}
		
		double preuRetornar = preu - ( preu * 0.3 );
		
		log.debug( "getPreuReservaRetornar() - Fi [preu=" + preuRetornar + "]" );
		return preuRetornar;
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.ReservaService#deleteReserva(java.lang.String)
	 */
	@Override
	public void deleteReserva(String codiReserva) throws Exception {
		log.debug( "deleteReserva() - Inici [codiReserva=" + codiReserva + "]" );
		ReservaVO reserva = reservaDAO.selectReserva(codiReserva);
		
		VolVO volAnada = volService.getVolByNumVol( reserva.getVolAnada() );
		VolVO volTornada = null;
		
		if( null != reserva.getVolTornada() ){
			volTornada = volService.getVolByNumVol( reserva.getVolTornada() );
		}
		
		List<PassatgerVO> passatgersPaguen = passatgerService.getAllPassatgersPaguen( codiReserva );
		List<PassatgerVO> totsPassatgers = passatgerService.getAllPassatgers( codiReserva );
		PassatgerVO contacte = passatgerService.getContacteByCodi( codiReserva );
		
		SessionFW sessio = null;	
		try{
			// Comencem la transacció de crear la reserva
			sessio = reservaDAO.openTransaccio( false );
			
			//Fem update de les places dels vols
			volAnada.setNumPlaces( volAnada.getNumPlaces() + passatgersPaguen.size() );
			//Actualitzem les places de vol
			volService.updateVol( sessio, volAnada );
			
			if( volTornada != null ){
				//Fem update de les places dels vols
				volTornada.setNumPlaces( volTornada.getNumPlaces() + passatgersPaguen.size() );
				//Actualitzem les places de vol
				volService.updateVol( sessio, volTornada );
			}
			
			// Esborrem la reserva
			reservaDAO.deleteReserva( sessio, reserva );
			
			//Esborrem els passatgers
			passatgerService.deletePassatger( sessio, contacte );
			
			for( PassatgerVO aux : totsPassatgers ){
				passatgerService.deletePassatger( sessio, aux );
			}
			
			// Si tot ha anat b fem el commit i guardem tots els canvis
			sessio.getTransactionFW().commit();
		}finally{
			reservaDAO.closeTransaccio(sessio);
		}
		
		log.debug( "deleteReserva() - Fi" );
	}
	
	// Mètodes privats

	/**
	 * 
	 */
	private List<String> createListNoms(List<PassatgerVO> passatgers) {
		List<String> out = new ArrayList<String>();
		
		for( PassatgerVO aux : passatgers ){
			out.add( aux.getNom() + " " + aux.getCognoms() );
		}
		
		return out;
	}

	/**
	 * 
	 */
	private ContacteVO createContacteVO(PassatgerVO contacte) {
		ContacteVO out = new ContacteVO();
		
		out.setTitol( contacte.getTitol() );
		out.setNom( contacte.getNom() );
		out.setCognoms( contacte.getCognoms() );
		out.setAdreca( contacte.getAdreca() );
		out.setPoblacio( contacte.getPoblacio() );
		out.setProvincia( contacte.getProvincia() );
		out.setCodiPostal( contacte.getCpostal() );
		out.setPais( contacte.getPais() );
		out.setTelf( String.valueOf( contacte.getTelf1() ) );
		out.setTelf2( String.valueOf( contacte.getTelf2() ) );
		out.setEmail( contacte.getEmail() );
		
		return out;
	}

	/**
	 * 
	 * @param numReserva
	 * @return
	 */
	private ReservaVO createReservaVO(String numReserva, ConsultaVO consultaVO) {
		ReservaVO out = new ReservaVO();
		
		out.setCodiReserva( numReserva );
		out.setVolAnada( consultaVO.getNumVolAnada() );
		if( consultaVO.getTipusViatge() == CatWingsConstants.VIATGE_ANADA_TORNADA ){
			out.setVolTornada( consultaVO.getNumVolTornada() );
		}
		out.setPassatger( numReserva );
		out.setPlaces( consultaVO.getNumAdults() + consultaVO.getNumNens() + consultaVO.getNumInfants() );
		out.setPagada( true );
		
		return out;
	}

	/**
	 * 
	 * @param aux
	 * @param numReserva
	 * @return
	 */
	private PassatgerVO createPassatgerVOFromInfant(InfantVO aux, String numReserva) {
		PassatgerVO out = new PassatgerVO();
		
		out.setCodiPassatger(numReserva);
		out.setNom( aux.getNom() );
		out.setCognoms( aux.getCognoms() );
		out.setDataNaixement( aux.getDataNaixement() );
		out.setAmbqui( aux.getViatgeAmb() );
		out.setTipus( CatWingsConstants.PASSATGER_TIPUS_INFANT );
		
		return out;
	}

	/**
	 * 
	 * @param aux
	 * @param numReserva
	 * @return
	 */
	private PassatgerVO createPassatgerVOFromNen(NenVO aux, String numReserva) {
		PassatgerVO out = new PassatgerVO();
		
		out.setCodiPassatger(numReserva);
		out.setNom( aux.getNom() );
		out.setCognoms( aux.getCognoms() );
		out.setDataNaixement( aux.getDataNaixement() );
		out.setTipus( CatWingsConstants.PASSATGER_TIPUS_NEN );
		
		return out;
	}

	/**
	 * 
	 * @param aux
	 * @param numReserva
	 * @return
	 */
	private PassatgerVO createPassatgerVOFromAdult(AdultVO aux,	String numReserva) {
		PassatgerVO out = new PassatgerVO();
		
		out.setCodiPassatger(numReserva);
		out.setTitol( aux.getTitol() );
		out.setNom( aux.getNom() );
		out.setCognoms( aux.getCognoms() );
		out.setDataNaixement( aux.getDataNaixement() );
		out.setTipus( CatWingsConstants.PASSATGER_TIPUS_ADULT );
		
		return out;
	}

	/**
	 * 
	 * @param contacte
	 * @return
	 */
	private PassatgerVO createPassatgerContacte(ContacteVO contacte, String numReserva) {
		PassatgerVO out = new PassatgerVO();
		
		out.setCodiPassatger(numReserva);
		out.setTitol( contacte.getTitol() );
		out.setNom( contacte.getNom() );
		out.setCognoms( contacte.getCognoms() );
		out.setAdreca( contacte.getAdreca() );
		out.setPoblacio( contacte.getPoblacio() );
		out.setProvincia( contacte.getProvincia() );
		out.setCpostal( contacte.getCodiPostal() );
		out.setPais( contacte.getPais() );
		if( contacte.getTelf() != null && !"".equals( contacte.getTelf() ) ){
			out.setTelf1( Integer.valueOf( contacte.getTelf() ) );
		}
		if( contacte.getTelf2() != null && !"".equals( contacte.getTelf2() ) ){
			out.setTelf2( Integer.valueOf( contacte.getTelf2() ) );
		}
		out.setEmail( contacte.getEmail() );
		out.setTipus( CatWingsConstants.PASSATGER_TIPUS_CONTACTE );
		
		return out;
	}
}
