package edu.uoc.pfc.lcamps.webapp.catwings.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.dao.VolDAO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.AeroportVO;
import edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO;
import edu.uoc.pfc.lcamps.webapp.catwings.constants.CatWingsConstants;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO;
import edu.uoc.pfc.lcamps.webapp.catwings.pojo.VolResultVO;
import edu.uoc.pfc.lcamps.webapp.catwings.service.AeroportService;
import edu.uoc.pfc.lcamps.webapp.catwings.service.VolService;

/**
 * Classe que implementa la interfície que gestiona el vol
 * @author Llorenç Camps 
 */
public class VolServiceImpl implements VolService {
	
	private static final Logger log = Logger.getLogger( VolServiceImpl.class );
	
	private AeroportService aeroportService;
	
	private VolDAO volDAO;
	
	/**
	 * @param aeroportService the aeroportService to set
	 */
	public void setAeroportService(AeroportService aeroportService) {
		this.aeroportService = aeroportService;
	}

	/**
	 * @return the aeroportService
	 */
	public AeroportService getAeroportService() {
		return aeroportService;
	}
	
	/**
	 * @return the volDAO
	 */
	public VolDAO getVolDAO() {
		return volDAO;
	}

	/**
	 * @param volDAO the volDAO to set
	 */
	public void setVolDAO(VolDAO volDAO) {
		this.volDAO = volDAO;
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.VolService#getInfoLlistatVols(edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO)
	 */
	@Override
	public VolResultVO getInfoLlistatVols(ConsultaVO consultaVO) {
		log.debug( "getInfoLlistatVols - Init" );
		VolResultVO out = new VolResultVO();
		
		int numPlaces = consultaVO.getNumAdults() + consultaVO.getNumNens();
				
		String nomAeroportSortida = getNomAeroport( consultaVO.getAeroportSortida() );
		String nomAeroportArribada = getNomAeroport( consultaVO.getAeroportArribada() );
		
		try {
			// Primer obtenim els vols d'anada
			List<VolVO> volsAnada = volDAO.selectVolPerData( consultaVO.getAeroportSortida(), consultaVO.getAeroportArribada(), consultaVO.getDataSortida(), numPlaces );
			// Actualitzem el nom de l'origen / desti
			updateOrigenDesti( volsAnada, nomAeroportSortida, nomAeroportArribada );
			out.setVolsAnada( volsAnada );
		} catch (DBException e) {
			log.debug( "getInfoLlistatVols - Error:" + e.getMessage() );
			out.setVolsAnada( new ArrayList<VolVO>() );
		}
		
		// Si busca anada i tornada consultarem els vols de tornada
		if( consultaVO.getTipusViatge() == CatWingsConstants.VIATGE_ANADA_TORNADA ){
			try {
				List<VolVO> volsTornada = volDAO.selectVolPerData( consultaVO.getAeroportArribada(), consultaVO.getAeroportSortida(), consultaVO.getDataTornada(), numPlaces );
				// Actualitzem el nom de l'origen / desti
				updateOrigenDesti( volsTornada, nomAeroportArribada, nomAeroportSortida );
				out.setVolsTornada( volsTornada );
			} catch (DBException e) {
				log.debug( "getInfoLlistatVols - Error:" + e.getMessage() );
				out.setVolsTornada( new ArrayList<VolVO>() );
			}
		}
		
		log.debug( "getInfoLlistatVols - Fi" );
		return out;
	}
		
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.VolService#getInfoVols(edu.uoc.pfc.lcamps.webapp.catwings.pojo.ConsultaVO)
	 */
	@Override
	public VolResultVO getInfoVols(ConsultaVO consultaVO) {
		log.debug( "getInfoVols - Init" );
		VolResultVO out = new VolResultVO();
			
		String nomAeroportSortida = getNomAeroport( consultaVO.getAeroportSortida() );
		String nomAeroportArribada = getNomAeroport( consultaVO.getAeroportArribada() );
				
		try {
			// Primer obtenim els vol d'anada
			List<VolVO> volsAnada = new ArrayList<VolVO>();
			volsAnada.add( this.getVolByNumVol( consultaVO.getNumVolAnada() ) );
			// Actualitzem el nom de l'origen / desti
			updateOrigenDesti( volsAnada, nomAeroportSortida, nomAeroportArribada );
			out.setVolsAnada( volsAnada );
		} catch (DBException e) {
			log.debug( "getInfoVols - Error:" + e.getMessage() );
			out.setVolsAnada( new ArrayList<VolVO>() );
		}
		
		// Si busca anada i tornada consultarem els vols de tornada
		if( consultaVO.getTipusViatge() == CatWingsConstants.VIATGE_ANADA_TORNADA ){
			try {
				List<VolVO> volsTornada = new ArrayList<VolVO>();
				volsTornada.add( this.getVolByNumVol( consultaVO.getNumVolTornada() ) );
				// Actualitzem el nom de l'origen / desti
				updateOrigenDesti( volsTornada, nomAeroportArribada, nomAeroportSortida );
				out.setVolsTornada( volsTornada );
			} catch (DBException e) {
				log.debug( "getInfoVols - Error:" + e.getMessage() );
				out.setVolsTornada( new ArrayList<VolVO>() );
			}
		}
		
		log.debug( "getInfoVols - Fi" );
		return out;
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.VolService#getVolByNumVol(int)
	 */
	@Override
	public VolVO getVolByNumVol( int numVol ) throws DBException{
		log.debug( "getVolByNumVol - Init" );		
		return volDAO.selectVol( numVol );
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.VolService#getNumPlacesDisponibles(int)
	 */
	@Override
	public int getNumPlacesDisponibles( int numVol ){
		log.debug( "getNumPlacesDisponibles - Init" );
		try {
			return volDAO.selectNumPlacesVol(numVol);
		} catch (DBException e) {
			log.debug( "getNumPlacesDisponibles - Error:" + e.getMessage() );
			return 0;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.VolService#updateVol(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO)
	 */
	@Override
	public void updateVol( SessionFW sessio, VolVO vol ) throws DBException{
		log.debug( "updateVol - Init" );
		volDAO.updateVol( sessio, vol );
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.webapp.catwings.service.VolService#deleteVol(edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW, edu.uoc.pfc.lcamps.webapp.catwings.bd.entity.VolVO)
	 */
	@Override
	public void deleteVol( SessionFW sessio, VolVO vol ) throws DBException{
		log.debug( "deleteVol - Init" );
		volDAO.deleteVol( sessio, vol );
	}
	
	
	public VolVO getVolUpdateNomAeroportByNumVol( int numVol ) throws DBException{
		log.debug( "getVolUpdateNomAeroportByNumVol - Init" );
		VolVO volVO = volDAO.selectVol( numVol );
		
		String nomAeroportSortida = getNomAeroport( volVO.getOrigen() );
		String nomAeroportArribada = getNomAeroport( volVO.getDesti() );
		
		volVO.setOrigen( nomAeroportSortida );
		volVO.setDesti( nomAeroportArribada );
		log.debug( "getVolUpdateNomAeroportByNumVol - Fi" );
		return volVO;
	}

	/*
	 * Mètodes privats
	 */
	private String getNomAeroport( String codiIata ){
		String nomAeroport = codiIata;
		try {
			AeroportVO aeroportSortida = aeroportService.getAeroport( codiIata );
			if( aeroportSortida != null ) nomAeroport = aeroportSortida.getCiutat() + " (" + aeroportSortida.getCodiIata() + ")"; 
		} catch (DBException e) {
			log.debug( "getNomAeroport - Error:" + e.getMessage() );
		}
		return nomAeroport;
	}
	
	private void updateOrigenDesti(List<VolVO> vols, String nomAeroportSortida, String nomAeroportArribada) {
		for( VolVO aux : vols ){
			aux.setOrigen( nomAeroportSortida );
			aux.setDesti( nomAeroportArribada );
		}
	}
}
