package edu.uoc.pfc.lcamps.webapp.catwings.spring;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import edu.uoc.pfc.lcamps.webapp.catwings.service.AeroportService;
import edu.uoc.pfc.lcamps.webapp.catwings.service.PassatgerService;
import edu.uoc.pfc.lcamps.webapp.catwings.service.ReservaService;
import edu.uoc.pfc.lcamps.webapp.catwings.service.VolService;

/**
 * Classe que serveix per gestionar els beans del context d'spring per obtenir els diferents serveis
 * @author Llorenç Camps 
 */
public class CatWingsContext {
	
	  private static final String AEROPORT_SERVICE = "aeroportService"; 
	  private static final String VOL_SERVICE = "volService";
	  private static final String RESERVA_SERVICE = "reservaService";
	  private static final String PASSATGER_SERVICE = "passatgerService";
	
	  private static final String CATWINGS_CONTEXT_LOCATION = "classpath:uoc-catwings-context.xml";
	  
	  private static CatWingsContext instance;

	  private ApplicationContext springContext;

	  public static synchronized CatWingsContext getInstance() {
	    if( instance == null ){
	      instance = new CatWingsContext();
	      instance.init();
	    }
	    return instance;
	  }
	
	  /*
	   *  Serveis
	   */
	  
	  public AeroportService getAeroportService(){
		return (AeroportService) this.springContext.getBean( AEROPORT_SERVICE );
	  }
	  
	  public VolService getVolService(){
	    return (VolService) this.springContext.getBean( VOL_SERVICE );
	  }
	  
	  public ReservaService getReservaService(){
		return (ReservaService) this.springContext.getBean( RESERVA_SERVICE );
	  }
	  
	  public PassatgerService getPassatgerService(){
		return (PassatgerService) this.springContext.getBean( PASSATGER_SERVICE );
	  }
	  
	  // Mètodes privats
	  /**
	   * Instancia el context utilitzant els fitxers a on es declaren els beans 
	   */
	  private void init() {
	    this.springContext = new ClassPathXmlApplicationContext( CATWINGS_CONTEXT_LOCATION );
	  }
}