package edu.uoc.pfc.lcamps.persistenciaFW.utils;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Date;
import java.util.Iterator;

import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.GeneratorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;
import edu.uoc.pfc.lcamps.persistenciaFW.generator.GeneratorColumnDBFW;
import edu.uoc.pfc.lcamps.persistenciaFW.generator.GeneratorTableDBFW;

/**
 * Classe que conté les funcions d'utilitats que s'utilitza en la classe de generació dels beans
 * @author Llorenç Camps
 * 13/12/2013
 */
public class GeneratorUtils {
	
	/**
	 * Donat el tipus de driver (MYSQL, ORACLE,...) retorna la classe que l'implementa
	 * @param idDriver
	 * @return
	 */
	public static String getClassNameFromDriver(String idDriver){
		return getClassNameFromDriver( idDriver, GeneratorConstants.PREFIX_DEFAULT_PCKG, GeneratorConstants.PREFIX_DEFAULT_CLASS, GeneratorConstants.SUFIX_DEFAULT_CLASS );
	}
	
	/**
	 * Donat el tipus de driver (MYSQL, ORACLE,...) retorna la classe que l'implementa
	 * @param idDriver
	 * @param prefixPackage
	 * @param prefixClass
	 * @param sufixClass
	 * @return
	 */
	public static String getClassNameFromDriver(String idDriver, String prefixPackage, String prefixClass, String sufixClass){
		String id = idDriver.toLowerCase().substring( 0,1 ).toUpperCase().concat( idDriver.toLowerCase().substring( 1 ) );
		return prefixPackage.concat( idDriver.toLowerCase() ).concat( GeneratorConstants.DOT ).concat( prefixClass ).concat( id ).concat( sufixClass );
	}
	
	/**
	 * 
	 * @param out
	 * @param paramString
	 * @throws IOException
	 */	
	public static void print(BufferedWriter out, String paramString)throws IOException {
		out.write( paramString );
	}
	
	/**
	 * Escriu la capçalera del fitxers de BBDD
	 * @param out
	 * @param gd
	 * @throws DBException
	 */
	public static void writeHeader(BufferedWriter out, GeneratorTableDBFW genTable )throws DBException{
		printBuffer( out, "", 0 );
		printBuffer( out, "package " + genTable.getPackageName() + ";", 0 );
		printBuffer( out, "", 0 );
		printBuffer( out, "import java.sql.*;", 0 );
		printBuffer( out, "import edu.uoc.pfc.lcamps.persistenciaFW.admin.annotations.*;", 0 );
		printBuffer( out, "import java.io.Serializable;", 0 );
		printBuffer( out, "", 0 );
		printBuffer( out, "/**", 0);
		printBuffer( out, " * Taula: " + genTable.getTableName(), 0 );
		printBuffer( out, " * Classe: " + genTable.getClassName(), 0 );
		printBuffer( out, " * --------------------------------------------------", 0 );
		printBuffer( out, " * Versió de la base de dades : " + genTable.getMetadataVersion(), 0 );
		printBuffer( out, " * Nom del driver : " + genTable.getMetadataDriverName(), 0 );
		printBuffer( out, " * Data generació : " + new Date(), 0 );
		printBuffer( out, " * --------------------------------------------------", 0 );
		printBuffer( out, " */", 0 );
		    
		// Indiquem l'annotacio de la taula
		printBuffer( out, "@Entity", 0 );
		printBuffer( out, "@Table( name=\"" + genTable.getTableName() + "\")", 0 );
		printBuffer( out, "public class " + genTable.getClassName() + " implements Serializable {", 0 );
		printBuffer( out, "" );
		printBuffer( out, "private static final long serialVersionUID = 1L;" );
		printBuffer( out, "" );
		printBuffer( out, "public static int numFields = " + genTable.getNumFields() + ";" );
		printBuffer( out, "" );
			    
		// Fem el print de les variables de la classe
		printBuffer( out, " // Atributs de la classe" );
		for( GeneratorColumnDBFW aux : genTable.getColumns() ){
			printBuffer( out, aux.getJavaType() + " " + StringUtilsFW.lowerCamelCase( aux.getName() )+ "; " );
		}
		printBuffer( out, "" );
			    
		writeConstructor( out, genTable.getClassName() );
	}
	
	/**
	 * Generera el constructor 
	 * @param out
	 * @param nomClasse
	 * @throws DBException
	 */
	public static void writeConstructor( BufferedWriter out, String nomClasse )throws DBException{
		// Constructor buit
		printBuffer( out, "public " + nomClasse + "() {}" );
		printBuffer( out, "" );
	}
	
	/**
	 * Genera el constructor amb tots els atributs 
	 * @param out
	 * @param gd
	 * @throws DBException
	 */
	public static void writeConstructor(BufferedWriter out, GeneratorTableDBFW tableDB ) throws DBException{
		printBuffer( out, "public " + tableDB.getClassName() + "(" , false );
	    Iterator<GeneratorColumnDBFW> iter = tableDB.getColumns().iterator();
	    GeneratorColumnDBFW columnData;
	    while( iter.hasNext() ){
	    	columnData = iter.next();
	    	printBuffer( out, columnData.getJavaType() + " " + StringUtilsFW.lowerCamelCase( columnData.getName().toLowerCase() ) 
	    			+ ( iter.hasNext() ? "," : ""), 0, false );
	    }
	    printBuffer( out, ") {" );
	    iter = tableDB.getColumns().iterator();
	    while( iter.hasNext() ){
	    	columnData = iter.next();
	    	printBuffer( out, "    this." + StringUtilsFW.lowerCamelCase( columnData.getName().toLowerCase() ) 
	    			+ " = " + StringUtilsFW.lowerCamelCase( columnData.getName().toLowerCase() ) + ";" );
	    }
	    printBuffer( out, "  }" );
	}
	
	/**
	 * Escriu els getters && Setters
	 * @param out
	 * @param genTable
	 * @throws DBException
	 */
	public static void writeSetGet(BufferedWriter out, GeneratorTableDBFW genTable )throws DBException{
		for( GeneratorColumnDBFW aux : genTable.getColumns() ){
			printBuffer( out, "" );
			writeSetGetColumn( out, aux );
		}
	}
	
	/**
	 * Sobreescriu el mètode toString
	 * @param out
	 * @param genTable
	 * @throws DBException
	 */
	public static void writeToString(BufferedWriter out, GeneratorTableDBFW genTable )throws DBException{
		printBuffer( out, "" );
		printBuffer( out, " /*" );
		printBuffer( out, "  * (non-Javadoc) " );
		printBuffer( out, "  * " );
		printBuffer( out, "  * @see java.lang.Object#toString()" );
		printBuffer( out, "  */" );
		printBuffer( out, "  @Override" );
		printBuffer( out, "  public String toString() {" );
		printBuffer( out, "    return " + constructToString( genTable ) );
		printBuffer( out, "}" );
		printBuffer( out, "" );
	}
	
	/**
	 * Generem el final del fitxer
	 * @param out
	 * @throws BDException
	 */
	public static void writeEnd( BufferedWriter out ) throws DBException {
		try {
			out.write( "}" );
			out.flush();
		}catch( Exception e) {
			throw new DBException( ErrorConstants.ERROR_GENERATE_BEANS_CODI, ErrorConstants.ERROR_GENERATE_BEANS_MSG + e.getMessage(), e );
		}
	}
	
	// Mètodes privats
	
	/*
	 * 
	 */
	private static void printBuffer(BufferedWriter out, String text, boolean b) throws DBException {
		printBuffer( out, text, 1, b );
	}
	
	/*
	 * 
	 */
	private static void printBuffer(BufferedWriter out, String text) throws DBException {
		printBuffer( out, text, 1, true );
	}
	
	/*
	 * 
	 */
	private static void printBuffer(BufferedWriter out, String text, int tab) throws DBException {
		printBuffer( out, text, tab, true );
	}
	
	/*
	 * 
	 */
	private static void printBuffer(BufferedWriter out, String text, int tab, boolean ret) throws DBException {
		try {
			String prefix = "";
			for( int i=0; i < tab; i++ ){
				prefix += "\t";
			}
			String sufix = ( ret == true) ? "\n" : "";
		    out.write( prefix + text + sufix );
		}catch( Exception e ){
			throw new DBException( ErrorConstants.ERROR_GENERATE_BEANS_CODI, ErrorConstants.ERROR_GENERATE_BEANS_MSG + e.getMessage(), e );
		}
	}
	
	/**
	 * Genera els getters && Setters
	 * @param out
	 * @param columna
	 * @throws BDException
	 */
	public static void writeSetGetColumn(BufferedWriter out, GeneratorColumnDBFW genColumn ) throws DBException{
		try{
			String str1 = StringUtilsFW.lowerCamelCase( genColumn.getName().toLowerCase() );
			String str2 = StringUtilsFW.lowerCamelCase( genColumn.getName().toLowerCase() );
			str2 = str2.substring( 0, 1 ).toUpperCase() + str2.substring( 1, str2.length() );
			    
			// Generate setter
			printBuffer( out, "  /**" );
			printBuffer( out, "   * Sets del valor per " + str1 );
			printBuffer( out, "   * @param " + str1 );
			printBuffer( out, "   */" );
			printBuffer( out, "  public void set" + str2 + "(" + genColumn.getJavaType() + " " + str1 + ") {" );
			printBuffer( out, "     this." + str1 + "=" + str1 + ";" );
			printBuffer( out, "  }" );
			printBuffer( out, "" );
			    
			// Generate getter
			printBuffer( out, "  /**");
			printBuffer( out, "   * Gets del valor per " + str1 );
			printBuffer( out, "   */" );
			
			// Afegim l'annotacio			    
			if( genColumn.isAutoincrement() ){
			    String idAnnotation = "  @Id(name=\"" + genColumn.getName() + "\", type=\"" + genColumn.getNameType() + "\")";
			    printBuffer( out, idAnnotation );
			} 
			
			String annotationColumn =! genColumn.isAutoincrement() ? "  @Column(name=\"" + genColumn.getName() + "\", type=\"" 
					+ genColumn.getNameType() + "\", nullable=\"" + (genColumn.isNullable() ? "SI" : "NO" ) + "\")":"@Sequence";
			printBuffer( out, annotationColumn );
			printBuffer( out, "  public " + genColumn.getJavaType() + " get" + str2 + "() {" );
			printBuffer( out, "    return " + str1 + ";" );
			printBuffer( out, " }" );

		}catch(Exception e){
			throw new DBException( ErrorConstants.ERROR_GENERATE_BEANS_CODI, ErrorConstants.ERROR_GENERATE_BEANS_MSG + " (getters/setters) : " + e.getMessage(), e );
		}
	}
	
	/**
	 * Construeix el toString amb els atributs
	 * @param genTable
	 * @return
	 */
	private static String constructToString(GeneratorTableDBFW genTable) {
		String out = "\"" + genTable.getClassName() + " [";
		int i = 0;
		for( GeneratorColumnDBFW aux : genTable.getColumns() ){
			String nomColumna = StringUtilsFW.lowerCamelCase( aux.getName().toLowerCase() );
			if( i != 0 )  out = out.concat( " + \", " );
			out = out.concat( nomColumna ).concat( "=\" + " ).concat( nomColumna );			
			i++;
		}
		out = out.concat( " + \"]\";" );
		return out;
	}
}